<?php
use Softneta\MedDream\Core\ActionFilter;
use Softneta\MedDream\Core\Audit;
use Softneta\MedDream\Core\Backend;
use Softneta\MedDream\Core\Branding;
use Softneta\MedDream\Core\Constants;
use Softneta\MedDream\Core\DICOM\DicomTagParser;
use Softneta\MedDream\Core\DicomTags;
use Softneta\MedDream\Core\HttpUtils;
use Softneta\MedDream\Core\Jobs;
use Softneta\MedDream\Core\Logging;
use Softneta\MedDream\Core\PresentationStateHandler;
use Softneta\MedDream\Core\SR;
use Softneta\MedDream\Core\Study;
use Softneta\MedDream\Core\System;
use Softneta\MedDream\Core\TagResults;
use Softneta\MedDream\Core\ProductInfo;
use Softneta\MedDream\Core\UpdateChecker;
use Softneta\MedDream\Core\HeaderData;

define('PATH_TO_ROOT', __DIR__ . DIRECTORY_SEPARATOR);
$moduleName = basename(__FILE__); /* for logging */
$basedir = __DIR__; /* for meddream_thumbnail etc */

ini_set('memory_limit', '512M');

require(PATH_TO_ROOT . 'sharedData.php');
require_once(PATH_TO_ROOT . 'autoload.php');
include_once(PATH_TO_ROOT . 'dcmsys/quidoSearch.php');

ini_set('session.cookie_httponly', 1);		/* for the PHPSESSID cookie */
session_start();

$log = null;
$backend = null;
$constants = new Constants();
$cmd = HttpUtils::getValue($_REQUEST, 'cmd', '');
//fix search sending json
$jsonData = array();
if (empty($cmd))
{
	$dump = @file_get_contents('php://input');
	$log = getLogging();
	$log->asDump('Received data:', $dump);
	$array = @json_decode($dump, true);
	if (!is_null($array))
	{
		$cmd = HttpUtils::getValue($array, 'cmd', '');
		if (!empty($cmd))
		{
			$jsonData = $array;
			//just in case
			$jsonData = array_merge($jsonData, $_REQUEST);
		}
	}
}

switch ($cmd)
{
	case 'getSettings':
		{
			$log = getLogging();
			$log->asDump("begin $moduleName|$cmd");
			$backend = getBackend(array('Auth'));		/* 'Auth': onConnect() and System::addPrivilegies() */

			if (!$backend->authDB->isAuthenticated())
				$settings = array('error' => 'not authenticated');
			else
				$settings = getSettings($log, $backend, new System($backend));

			$log->asDump('Settings = ', $settings);
			echo safeEncode($settings);
			$log->asDump("end $moduleName|$cmd");
		}
		break;

	case 'SaveSettings':
		{
			$log = getLogging();
			$backend = getBackend(array(), false);

			if (!$backend->authDB->isAuthenticated())
				$return = array('error' => 'not authenticated');
			else
			{
				$return = array('error' => 'empty or bad settings');
				if (!empty($_REQUEST['data']))
				{
					$data = json_decode($_REQUEST['data'], true);
					$log->asDump('Settings to save = ', $data);
					if (!empty($data))
					{
						/**
						 * remove system configuration settings
						 */
						if (isset($data['privileges']))
							unset($data['privileges']);
						if (isset($data['debug']))
							unset($data['debug']);
						if (isset($data['qr']))
							unset($data['qr']);
						if (isset($data['toolLinks']))
							unset($data['toolLinks']);
						if (isset($data['share']))
							unset($data['share']);

						breakJavascript($data);

						$system = new System($backend);
						$return['error'] = $system->saveSettings(safeEncode($data));
					}
				}
			}
			echo safeEncode($return);
		}
		break;

	case 'setLanguage':
		{
			$lng = HttpUtils::getValueStrict($_REQUEST, 'lan');
			$log = getLogging();
			$log->asDump("begin $moduleName|$cmd(", $lng, ')');
			$backend = getBackend(array(), false);
			$system = new System($backend);
			$return = $system->updateLanguage($lng);

			/* the function above was meant for Flash and returns the whole
			   XML-format translation file. No need to waste traffic.
			 */
			if (isset($return['languages']))
				unset($return['languages']);

			echo safeEncode($return);
			$log->asDump("end $moduleName|$cmd");
		}
		break;

	case 'userIsAuthenticated':
		{
			$log = getLogging();
			$log->asDump("begin $moduleName|$cmd");
			$backend = getBackend(array(), false);
/*
			$url = '';
			if (!$backend->authDB->isAuthenticated())
			{

				if (!empty($_SERVER['HTTPS']) && ('on' == $_SERVER['HTTPS']))
					$url = 'https://';
				else
					$url = 'http://';

				$url .= $_SERVER['HTTP_HOST'] . str_replace("\\", "/", dirname(dirname($_SERVER['PHP_SELF'])));

				header('Cache-Control: no-cache');
				header('Pragma: no-cache');
				header('Location: ' . $url);
			}
*/

			$result = array();

			$result['clientId'] = '';
			$result['isAuthenticated'] = $backend->authDB->isAuthenticated();
			if ($result['isAuthenticated'] && isset($_SESSION['clientIdMD']))
				$result['clientId'] = $_SESSION['clientIdMD'];

			$log->asDump('returning: ', $result);
			HttpUtils::returnJSON($result, false);
			$log->asDump("end $moduleName|$cmd");
		}
		break;

	case 'connect':
		{
			$log = getLogging();
			$log->asDump("begin $moduleName|$cmd");
			$backend = getBackend(array('Auth'));		/* 'Auth': onConnect() and System::addPrivilegies() */

			$sys = new System($backend);
			$clientid = HttpUtils::getValue($_REQUEST, 'clientid', '');
			$rtn = $sys->connect($clientid);
			$rtn['settings'] = getSettings($log, $backend, $sys);
			unset($rtn['languages']);
			unset($rtn['qr']);

			$log->asDump('returning: ', $rtn);
			echo safeEncode($rtn);
			$log->asDump("end $moduleName|$cmd");
		}
		break;

	case 'getProductInformation':
		{
			$log = getLogging();
			$log->asDump("begin $moduleName|$cmd");

			$backend = getBackend(array('Auth'));
				/* 2nd parameter missing: must be connected to retrieve PacsOne privileges */
			$err = $backend->tr->configure();
			if (is_string($err) && $err)
				$log->asErr($err);
			$err = $backend->tr->load();
			if ($err)
				$log->asErr($err);
			$uc = new UpdateChecker(-1, null, null, $log, $constants);
			$sys = new System($backend, null, null, $log);
			$about = new ProductInfo($backend, $sys, $uc, new Branding(), $backend->tr, $log);

			$data = $about->collectData();

			$log->asDump('returning: ', $data);
			$log->asDump("end $moduleName|$cmd");
			if (is_string($data))
				HttpUtils::error($data);
			else
				HttpUtils::returnJSON($data);
		}
		break;

	case 'getReleaseNotes':
		HttpUtils::returnJSON(parseReleaseNotes());
		break;

	case 'system':
		{
			$sys = new System();

			$clientId = HttpUtils::getValueStrict($_REQUEST, 'id');
			$fileContents = str_replace(array("\n", "\r", "\t"), '',
				$sys->license($clientId));

			$fileContents = trim(str_replace('"', "'", $fileContents));

			$simpleXml = simplexml_load_string($fileContents);

			$simpleXml->addChild('connections', $sys->connections());

			$json = json_encode($simpleXml);
			echo $json;
		}
		break;

	case 'branding':
		{
			$sys = new System();
			$sys->addBranding($return);
			if (!empty($return['branding']))
				echo safeEncode($return['branding']);
			else
				echo '{}';
		}
		break;

	/*case 'loadSeriesData': //todo 5.0 patikrinti ar niekur nenaudojamas, jeigu ne i�mesti
		{
			$study = '';
			$studyUID = $_POST['study'];
			$st = new Study();
			if ($studyUID != "")
				$study = $st->getStudyList($studyUID);

			for ($i = 0; $i < $study['count']; $i++)
			{
				if ($study[$i]['id'] == $_POST['series'])
				{
					$series = $study[$i];
				}
			}
			echo json_encode($series);
		}
	break;*/

	case 'loadStudy':
		{
			$log = getLogging();
			$studyUID = "";
			$seriesUID = "";
			$imageUID = "";
			$log->asDump("begin $moduleName|$cmd");
			$flt = new ActionFilter($log);

			$auditAction = "";
			$auditObject = "";

			$backend = getBackend(array('Structure'));
			$cached = 0;
			$imageUids = array();
			$seriesUids = array();

			$cached = HttpUtils::getValue($_REQUEST, 'cached', 0);
			$studyUID = HttpUtils::getValue($_REQUEST, 'study', '');
			$seriesUID = HttpUtils::getValue($_REQUEST, 'series', '');
			$imageUID = HttpUtils::getValue($_REQUEST, 'image', '');

			$flt->getActionsInto($actions);
			$study = array('error' => '');
			if ($studyUID != "")
			{
				$study = $backend->pacsStructure->studyGetMetadata($studyUID, false, $cached == 1);
				if (!$cached)
				{
					$auditAction = 'OPEN STUDY';
					$auditObject = $studyUID;
				}
				$flt->validateAtStudyLevel('study', $studyUID, $actions, $study);
			}
			elseif ($seriesUID != "")
			{
				$seriesList = explode("|", $seriesUID);
				$study = $backend->pacsStructure->studyGetMetadataBySeries($seriesList);
				$log->asDump('getSeriesList: ', $study);

				$auditAction = 'OPEN SERIES';
				$auditObject = $seriesUID;

				$flt->validateAtStudyLevel('series', $seriesUID, $actions, $study);

			}
			elseif ($imageUID != "")
			{
				$imageList = explode("|", $imageUID);
				$study = $backend->pacsStructure->studyGetMetadataByImage($imageList, $cached == 2);
				$log->asDump('getImageList: ', $study);

				if (!$cached)
				{
					$auditAction = 'OPEN IMAGE';
					$auditObject = $imageUID;
				}
				$flt->validateAtStudyLevel('image', $imageUID, $actions, $study);
			}

			$audit = new Audit($auditAction);

			if (strlen($study['error']))
			{
				$protocol = 'HTTP/1.0';
				if (isset($_SERVER['SERVER_PROTOCOL']))
					$protocol = $_SERVER['SERVER_PROTOCOL'];
				header($protocol . ' 500 Internal Server Error');

				echo safeEncode(array('error' => $study['error']));
				$audit->log(false, $auditObject);
			}
			else
			{
				for ($i = 0; $i < $study['count']; $i++)
				{
					for ($j = 0; $j < $study[$i]['count']; $j++)
					{
						$ids = explode('*', $study[$i][$j]['id']);
						$imageUids[] = $ids[0];
						unset($study[$i][$j]['path']);
						changeKeyValue($study[$i][$j], 'bitsstored', 'bitsStored', 0);
						changeKeyValue($study[$i][$j], 'numframes', 'numFrames', 0);
						changeKeyValue($study[$i][$j], 'xfersyntax', 'xferSyntax', '');
					}
					$ids = explode('*', $study[$i]['id']);
					$seriesUids[] = $ids[0];
					$study[$i]['studyId'] = $study['uid'];
				}

				changeKeyValue($study, 'firstname', 'firstName', '');
				changeKeyValue($study, 'lastname', 'lastName', '');
				changeKeyValue($study, 'patientid', 'patientId', '');
				changeKeyValue($study, 'sourceae', 'sourceAE', '');
				changeKeyValue($study, 'studydate', 'studyDate', '');
				changeKeyValue($study, 'studytime', 'studyTime', '');

				/* cache IDs for security filters */
				$flt->markValidPatientId($study['patientId']);	/* in ?cmd=loadStudySearch */
				if ($flt->actionsDoExist($actions))
				{
					$flt->markValidSeriesUids($seriesUids);
					$flt->markValidImageUids($imageUids);
				}

				echo safeEncode($study);
				$log->asDump("end $moduleName|$cmd");
				$audit->log(true, $auditObject);
			}
		}
		break;

	case 'loadSeries': //todo 5.0 patikrinti ir i�mesti
		{
			$flt = new ActionFilter(getLogging());
			$flt->getActionsInto($actions);
			$study = '';
			$studyUID = HttpUtils::getValue($_POST, 'study', '');
			$backend = getBackend(array('Structure'));
			if ($studyUID != "")
			{
				$study = $backend->pacsStructure->studyGetMetadata($studyUID);
				if (!$flt->validateAtStudyLevel('study', $studyUID, $actions, $study))
					exit('Unauthorized access to object(s)');
			}
			$series = array();
			$seriesUID = HttpUtils::getValueStrict($_POST, 'series');
			if (isset($study['count']))
			{
				for ($i = 0; $i < $study['count']; $i++)
				{
					if ($study[$i]['id'] == $seriesUID)
					{
						$series = $study[$i];
					}
				}
			}
			echo json_encode($series);
		}
		break;

	case 'counts':
		{
			$log = getLogging();
			$log->asDump("begin $moduleName|$cmd");
			$backend = getBackend(array('Search'));
			$return = $backend->pacsSearch->getStudyCounts();
			echo json_encode($return);
			$log->asDump("end $moduleName|$cmd");
		}
		break;

	case 'loadStudySearch':
		{
			$log = getLogging();
			$log->asDump("begin $moduleName|$cmd ");
			$log->asDump('$jsonData:', $jsonData);

			$searchCriteria = array();
			$criteriaKeys = array(
				'id' => 'patientid',
				'name' =>'patientname',
				'accessionnum' => 'accessionnum',
				'description' => 'description',
				'sourceae' => 'sourceae',
			);
			$index = 0;
			if (!empty($jsonData['filter']))
			{
				for ($i = 0; $i < count($jsonData['filter']); $i += 2)
				{
					$criteriaName = '';
					if (array_key_exists($jsonData['filter'][$i], $criteriaKeys))
						$criteriaName = $criteriaKeys[$jsonData['filter'][$i]];
					else
						continue;

					$searchCriteria[$index] = array();
					$searchCriteria[$index]['name'] = $criteriaName;
					$searchCriteria[$index]['text'] = $jsonData['filter'][$i + 1];
					$index++;
				}
			}
			$addLiveStreamData = false;
			$selectedModality = array();
			if (!empty($jsonData['modality']))
			{
				$log->asDump('Modality filter received: ', $jsonData['modality']);
				if (count($jsonData['modality']) > 0)
				{
					$jsonData['modality'] = array_map('strtoupper', $jsonData['modality']);
					$allModality = array('CR', 'CT', 'DX', 'ECG', 'ES', 'IO',
										'MG', 'MR', 'NM','OT', 'PX', 'RF', 'RG',
										'SC', 'US', 'XA', 'XC', 'LIVE');
						//set searching modalities
						$modalities = array();
						foreach ($jsonData['modality'] as $tmpModality)
						{
							if (!empty($modalities[$tmpModality]))
								continue;
							
							//custom modality
							if ($tmpModality == 'LIVE')
								$addLiveStreamData = true;

							$modality = array();
							$modality['name'] = $tmpModality;
							$modality['selected'] = true;
							$modalities[$tmpModality] = $modality;
						}
						//add other modalities from $allModality
						$someIsNotSelected = false;
						foreach ($allModality as $oneOfAllModality)
						{
							if (!empty($modalities[$oneOfAllModality]))
								continue;

							$modality = array();
							$modality['name'] = $oneOfAllModality;
							$modality['selected'] = false;
							$modalities[$oneOfAllModality] = $modality;
							$someIsNotSelected = true;
						}
						//set custom attribute for new modalities
						$diff = array_diff(array_keys($modalities), $allModality);
						foreach ($diff as $modality)
						{
							if (!empty($modalities[$modality]))
								$modalities[$modality]['custom'] = true;
						}
						unset($diff);
						//there is some modalities that is not selected or
						//there is some new modalities
						if ($someIsNotSelected ||
							(count($modalities) != count($allModality)))
							$selectedModality = array_values($modalities);
						unset($modalities);
				}
			}
			else
				$addLiveStreamData = true;

			$log->asDump('Modality filter will be used: ', $selectedModality);

			$fromDate = HttpUtils::getValue($jsonData, 'startDate');
			$toDate = HttpUtils::getValue($jsonData, 'endDate');
			$listMax = 1000;

			$data = array(); 
			if (isset($_COOKIE["suid"]))
			{
				$quidoSearch = new guidoSearch();
				$data = $quidoSearch->getData($searchCriteria, $fromDate, $toDate,
					$selectedModality, $listMax);
			}
			else
			{
				$backend = getBackend(array('Search'));
				$actions = HttpUtils::getValue($jsonData, 'actions', '');
				if (!empty($actions))
				{
					$log->asDump('Actions filter will be used: ', $actions);
				}
				$flt = new ActionFilter($log);
				if ($flt->getActionsInto($actions2))
				{
					$err = $flt->check($actions, $actions2);
					if ($err)
					{
						$results = array(array('type' => 'error', 'description' => $err));
						$json = formJsonResult($results, 500);
						exit($json);
					}
				}
				$data = $backend->pacsSearch->findStudies($actions, $searchCriteria, $fromDate, $toDate,
					$selectedModality, $listMax);
			}
			
			if (!empty($data['error']))
			{
				$results = array(array(
					'type' => 'error',
					'description' => $data['error']
				));
				$json = formJsonResult($results, 500);
				exit($json);
			}

			$anotation = new PresentationStateHandler();
			$count = $data['count'];
			$tmp = array();
			for ($i = 0; $i < $count; $i++)
			{
				if (empty($data[$i]['uid']))
					continue;
				
				$item = array();
				$item['uid'] =  $data[$i]['uid'];
				if (array_key_exists('patientid', $data[$i]))
					$item['id'] = $data[$i]['patientid'];
				if (array_key_exists('patientname', $data[$i]))
					$item['name'] = $data[$i]['patientname'];
				if (array_key_exists('accessionnum', $data[$i]))
					$item['accessionNumber'] = $data[$i]['accessionnum'];
				if (array_key_exists('modality', $data[$i]))
					$item['modality'] =  $data[$i]['modality'];
				if (array_key_exists('description', $data[$i]))
					$item['description'] = $data[$i]['description'];
				if (array_key_exists('datetime', $data[$i]))
					$item['createDatetime'] = $data[$i]['datetime'];
				if (array_key_exists('received', $data[$i]))
					$item['receiveDatetime'] = $data[$i]['received'];
				if (array_key_exists('sourceae', $data[$i]))
					$item['sourceae'] = $data[$i]['sourceae'];
				$isRead = false;
				if (array_key_exists('reviewed', $data[$i]))
					$isRead = (bool)$data[$i]['reviewed'];
				$item['isRead'] = $isRead;
				$haveReport = false;
				$availableReport = false;
				if (array_key_exists('notes', $data[$i]))
				{
					$note = (int)$data[$i]['notes'];
					if ($note == 1)
					{
						$haveReport = true;
						$availableReport = true;
					}
					else
						if ($note == 0)
							$availableReport = true;
				}
				$item['haveReport'] = $haveReport;
				$item['availableReport'] = $availableReport;
				
				$haveAnnotation = (bool)$anotation->hasAnnotations($data[$i]['uid']);
				$item['haveAnnotation'] = $haveAnnotation;
					
				$tmp[] = $item;
			}
			unset($data);

			if ($addLiveStreamData)
				updateWithLiveData($tmp, $searchCriteria);

			$results = array(
				'recordsFiltered' => count($tmp),
				'studies' => $tmp);

			echo formJsonResult($results, 200);
			$log->asDump("end $moduleName|$cmd");
		}
		break;

	case 'latestversion':
		$sclass = new System();
		echo safeEncode($sclass->latestVersion());
		break;

	case 'getForwardAEList':
		$AElist = new Study();
		echo json_encode($AElist->getForwardAEList());
		break;

	case 'forwardStudies':
		$uid = HttpUtils::getValueStrict($_REQUEST, 'uid');
		$aet = HttpUtils::getValueStrict($_REQUEST, 'aet');
		$study = new Study();
		echo json_encode($study->forward($uid, $aet, ''));
		break;

	case 'getForwardStatus':
		$study = new Study();
		$return = array('error' => '', 'status' => '');
		$jobId = HttpUtils::getValue($_REQUEST, 'jobId');
		if ($jobId > -1)
		{
			$return['status'] = $study->forwardStatus($jobId);
		}
		else
		{
			$return['error'] = 'Missing job id';
		}
		echo safeEncode($return);
		break;
	case 'getVolumeSizes':
		include_once(PATH_TO_ROOT . 'export.php');
		$size = new Export();
		echo json_encode($size->getVolumeSizes());
		break;
	case 'exportStudies':
		include_once(PATH_TO_ROOT . 'export.php');
		$uid = HttpUtils::getValueStrict($_REQUEST, 'uid');
		$size = HttpUtils::getValue($_REQUEST, 'size', '');
		$export = new export();
		echo json_encode($export->media($uid, $size, ''));
		break;
	case 'exportStatus':
		include_once(PATH_TO_ROOT . 'export.php');
		$id = HttpUtils::getValueStrict($_REQUEST, 'id');
		$export = new export();
		echo json_encode($export->status($id));
		break;
	case 'exportDeleteTemp':
		include_once(PATH_TO_ROOT . 'export.php');
		$timestamp = HttpUtils::getValueStrict($_REQUEST, 'timestamp');
		$export = new export();
		echo json_encode($export->deleteTemp($timestamp));
		break;
	case 'getExportIso':
		include_once(PATH_TO_ROOT . 'export.php');
		$timestamp = HttpUtils::getValueStrict($_REQUEST, 'timestamp');
		$mediaLabel = HttpUtils::getValueStrict($_REQUEST, 'mediaLabel');
		$export = new export();
		echo json_encode($export->vol($timestamp, $mediaLabel));
		break;
	case 'getExportNotes':
		include_once(PATH_TO_ROOT . 'export.php');
		$timestamp = HttpUtils::getValueStrict($_REQUEST, 'timestamp');
		$uid = HttpUtils::getValueStrict($_REQUEST, 'uid');
		$export = new export();
		echo json_encode($export->notes($timestamp, $uid));
		break;

	case 'getImagePart':
		$log = getLogging();
		$log->asDump("begin $moduleName?cmd=$cmd");

		$uid = HttpUtils::getValueStrict($_REQUEST, 'uid');
		$log->asDump('$uid = ', $uid);

		$step = HttpUtils::getParam('step', 1);
		$log->asDump('$step = ', $step);

		$flt = new ActionFilter($log);
		$flt->getActionsInto($actions);
		$err = $flt->validateAtImageLevel($uid, $actions);
		if ($err)
			HttpUtils::error($err);

		try
		{
			$rawAdam7 = new RawAdam7($uid, buildMedDreamCmd(MEDDREAM_GETRAW_PROGRESSIVE), 7);
			header('Content-Type: application/octet-stream');
			$rawAdam7->printStep($step);
		}
		catch (RuntimeException $e)
		{
			HttpUtils::error($e->getMessage());
		}

		$log->asDump("end $moduleName?cmd=$cmd");
		break;

    case 'getImageLines':
        $log = getLogging();
        $log->asDump("begin $moduleName?cmd=$cmd");

        $uid = HttpUtils::getValueStrict($_REQUEST, 'uid');
        $log->asDump('$uid = ', $uid);

        $step = HttpUtils::getParam('step', 0);
        $log->asDump('$step = ', $step);

        $count = HttpUtils::getParam('count', 0);
        $log->asDump('$count = ', $count);

		$flt = new ActionFilter($log);
		$flt->getActionsInto($actions);
		$err = $flt->validateAtImageLevel($uid, $actions);
		if ($err)
			HttpUtils::error($err);

        try
        {
            $md = buildMedDreamCmd();
            $meta = $md->getMeta($uid);
        }
        catch (Exception $e)
        {
            HttpUtils::error($e->getMessage());
        }

        $powStep = pow(2, $step);
        $readStep = pow(2, $step + 1 <= $count ? $step + 1 : $count);

        header('Content-Type: application/octet-stream');
        $md->printRaw($uid, 0, $powStep - 1, $readStep, true);

        $log->asDump("end $moduleName?cmd=$cmd");
        break;

	case 'getSupportedLanguages':
		$sclass = new System();
		echo safeEncode($sclass->getSupportedLanguages());
		break;

	case 'getCachedImageList':
		$studyUid = '';

		$log = getLogging();
		$studyUid = HttpUtils::getValueStrict($_POST, 'study');

		$backend = getBackend(array('Structure'));
		$study = $backend->pacsStructure->studyGetMetadata($studyUid, false, 1);

		$log->asDump('cmd:getCachedImageList, PacsStructure::studyGetMetadata: ' . var_export($study, true));

		$cached = array();
		$ni = 0;
		if (isset($study['count']))
			for ($i = 0; $i < $study['count']; $i++)
				for ($j = 0; $j < $study[$i]['count']; $j++)
				{
					$id = '';
					$path = '';
					if (isset($study[$i][$j]['id']))
						$id = fixId($study[$i][$j]['id']);

					if (isset($study[$i][$j]['path']))
						$path = fixId($study[$i][$j]['path']);

					if (empty($id) || empty($path))
						continue;

					$cached[] = array('id' => $id, 'path' => $path);
					$ni++;
				}

		$log->asDump("returning $ni entries for \$cached = ", $cached);

		unset($study);
		unset($studyUid);

		HttpUtils::returnJSON($cached);
		break;
	case 'getVideoData':
		{
			echo getVideoData();
		}
		break;
	case 'getInstanceTags':
		{
            header('Content-Type: application/json');
			$uid = HttpUtils::getValueStrict($_REQUEST, 'uid');
			echo safeEncode(getInstanceTags($uid));
		}
		break;
	case 'getInstanceMetadata':
		$uid = HttpUtils::getValueStrict($_REQUEST, 'uid');
		$flt = new ActionFilter(getLogging());
		$flt->getActionsInto($actions);
		$err = $flt->validateAtImageLevel($uid, $actions);
		if ($err)
			HttpUtils::error($err);
		HttpUtils::returnJSON(getInstanceMetadata($uid));
		break;

    case 'getSeriesMetadata':
		$studyUID = HttpUtils::getValueStrict($_REQUEST, 'studyUID');
		$seriesUID = HttpUtils::getValueStrict($_REQUEST, 'seriesUID');
        $backend = getBackend(array('Structure'));
        $flt = new ActionFilter(getLogging());
        $flt->getActionsInto($actions);
        $study = $backend->pacsStructure->studyGetMetadata($studyUID);
        $flt->validateAtStudyLevel('study', $studyUID, $actions, $study);

        if ($study['error'])
            HttpUtils::error($study['error']);

		set_time_limit(0);
		session_write_close();
        $results = array();
        for ($i = 0; $i < $study['count']; $i++) {
            $series = $study[$i];
            $parts = explode('*', $series['id']);
            if ($parts[0] !== $seriesUID)
                continue;
            for ($j = 0; $j < $series['count']; $j++) {
                $instance = $series[$j];
                $results[$instance['id']] = getInstanceMetadata($instance['id']);
            }
        }

        HttpUtils::returnJSON($results);
        break;

    case 'getInstancePixels':
		$uid = HttpUtils::getValueStrict($_REQUEST, 'uid');
        $frameNum = HttpUtils::getParam('frameNum', 0);

		$flt = new ActionFilter(getLogging());
		$flt->getActionsInto($actions);
		$err = $flt->validateAtImageLevel($uid, $actions);
		if ($err)
			HttpUtils::error($err);

        header('Content-Type: application/octet-stream');
        printInstancePixels($uid, $frameNum);

        break;

	case 'getVoiLut':
		if (!function_exists('meddream_get_voi_lut'))
			HttpUtils::returnJSON(array('error' => 'php_meddream version too old'));

		$uid = HttpUtils::getValueStrict($_REQUEST, 'uid');
		$name = HttpUtils::getValueStrict($_REQUEST, 'name');

		$backend = getBackend(array('Structure'));
		$st = $backend->pacsStructure->instanceGetMetadata($uid);
		if (strlen($st['error']))
			HttpUtils::returnJSON($st);
		$path = $st['path'];

		$vl = meddream_get_voi_lut(__DIR__, $path, $name);
		if ($vl['error'])
		{
			$vl['error'] = meddream_strerror($vl['error']);
			HttpUtils::returnJSON($vl);
		}
		else
			$vl['error'] = '';

		unset($vl['raw']);
		HttpUtils::returnJSON($vl);

		break;

	case 'getColorLut':
		if (!function_exists('meddream_get_color_lut'))
			HttpUtils::returnJSON(array('error' => 'php_meddream version too old'));

		$uid = HttpUtils::getValueStrict($_REQUEST, 'uid');

		$backend = getBackend(array('Structure'));
		$st = $backend->pacsStructure->instanceGetMetadata($uid);
		if (strlen($st['error']))
			HttpUtils::returnJSON($st);
		$path = $st['path'];

		$cl = meddream_get_color_lut(__DIR__, $path);
		if ($cl['error'])
		{
			$cl['error'] = meddream_strerror($cl['error']);
			HttpUtils::returnJSON($cl);
		}
		else
			$cl['error'] = '';

		unset($cl['rawR']);
		unset($cl['rawG']);
		unset($cl['rawB']);
		HttpUtils::returnJSON($cl);

		break;

	case 'saveAnnotationObject':
		{
			echo saveAnnotationObject();
		}
		break;
	case 'getAnnotationList':
		{
			echo getAnnotationList();
		}
		break;
	case 'getAnnotationObject':
		{
			echo getAnnotationObject();
		}
		break;
	case 'saveAnnotationJpeg':
		{
			echo saveAnnotationJpeg();
		}
		break;
	case 'getpathById':
		{
			$instanceID = HttpUtils::getValueStrict($_POST, 'instanceID');
			echo getPathById($instanceID);
		}
		break;
	case 'loadMpeg2':
		{
			$instanceID = HttpUtils::getValueStrict($_REQUEST, 'instanceID');
			echo loadMpeg2($instanceID);
		}

		break;
	case 'loadThumbnail':
		{
			$backend = getBackend(array('Structure', 'Preload'));
			$id = HttpUtils::getValue($_REQUEST, 'id', '');
			if (!$backend->authDB->isAuthenticated() || !empty($id))
			{
				echo safeEncode(array());
				exit();
			}
			$authDB = $backend->authDB;
///TODO
			$sql = "SELECT uuid , xfersyntax,  bitsstored, path FROM image WHERE image.uuid = '".$authDB->sqlEscapeString($id)."'";

			$rs = $authDB->query($sql);
			$r = $authDB->fetchNum($rs);

			if (is_array($r))
			{
				$study[0]['modality'] = '';
				$study[0][0]["id"] = $r[0];
				$study[0][0]["bitsstored"] = $r[2];
				$study[0][0]["xfersyntax"] =  $r[1];
				$study[0]["count"] = 10;
				$study[0][0]['path']  =  $r[3];

				echo 'data:image/png;base64,'.base64_encode( writeThumbnail(0, '', $study, 0, 0, 50));
			}
			exit;
		}

		break;
	case 'logPerformance':

		$log = getLogging();
		if ($log->isLoggingLevelEnabled(Logging::LEVEL_DEBUG))
		{
			$uid = HttpUtils::getValueStrict($_REQUEST, 'uid');
			$type = HttpUtils::getValueStrict($_REQUEST, 'type');
			if (!in_array($type, array('initiated', 'started', 'finished')))
				exit(safeEncode(array('error' => 'invalid time type')));

			$viewer = HttpUtils::getValueStrict($_REQUEST, 'viewer');
			$time = HttpUtils::getValueStrict($_REQUEST, 'time');

			$filename = PATH_TO_ROOT . 'log' . DIRECTORY_SEPARATOR . 'performance.csv';

			if (!file_exists($filename))
				file_put_contents($filename, "UID;Viewer;Type;Time\n", LOCK_EX | FILE_APPEND);

			file_put_contents($filename, "$uid;$viewer;$type;$time\n", LOCK_EX | FILE_APPEND);
			exit(safeEncode(array('ok' => 1)));
		}

		break;

	case 'dicomTags':
		$log = getLogging();
		$uid = HttpUtils::getValueStrict($_REQUEST, 'instanceID');
		$flt = new ActionFilter(getLogging());
		$flt->getActionsInto($actions);
		$err = $flt->validateAtImageLevel($uid, $actions);
		if ($err)
			HttpUtils::error($err);
		if (!empty($uid))
		{
			$backend = getBackend(array('Structure'));
			$tags = new DicomTags($backend, $log);
			$return = $tags->getTags($uid);
			echo safeEncode($return);
		}
		else
		{
			$log->asErr('missing instanceID');
			echo '{"error":"missing instanceID"}';
		}
		break;

	case 'formatSR':
		$uid = HttpUtils::getValueStrict($_REQUEST, 'uid');
		$flt = new ActionFilter(getLogging());
		$flt->getActionsInto($actions);
		$err = $flt->validateAtImageLevel($uid, $actions);
		if ($err)
			HttpUtils::error($err);

		$sr = new SR();
		$out = $sr->getHtml($uid);
		if (strlen($out['error']))
		{
			if ($out['error'] == 'not implemented')
			{
				$backend = new Backend(array(), false);
				$out['error'] = 'Display of SR objects is not implemented for PACS "' . $backend->pacs . '".';
			}
			echo 'Processing failed: ' . $out['error'];
		}
		else
			echo $out['html'];
		break;

	case 'ecg':
		header('Content-Type: application/json');
		echo safeEncode(getEcg());
		break;

	case 'ecgImg':
		try {
			$ecg = getEcg();
			$dpi = HttpUtils::getParam('dpi');
			$cols = HttpUtils::getParam('cols', 1);
			$mmms = HttpUtils::getParam('mmms', 25);
			$mmmV = HttpUtils::getParam('mmmV', 10);
			$img = \Softneta\MedDream\Core\ECG\ECGImage::create($ecg, $dpi, $mmms, $mmmV, $cols);

			header('Content-type: image/png');

			imagepng($img);
			imagedestroy($img);
		} catch (Exception $e) {
			exit('{"error":"' . $e->getMessage() . '"}');
		}

		break;

    case 'ecgInfo':

        header('Content-Type: application/json');

		$uid = HttpUtils::getValueStrict($_REQUEST, 'uid');
		$backend = getBackend(array('Structure'));
		if (!$backend->authDB->isAuthenticated())
            exit('{"error":"Not Authenticated"}');

        $path = getPathById($uid);
        $tags = DicomTagParser::parseFile($path, 4);

        $info = $tags[0x0040][0xb020][0xfffe][0xe000];

        $a = 0;
        $results = array();
        foreach ($info as $key => $item) {
            $group = $item[0x0040][0xa180]['data'][0];
            if ($group === 0 && isset($item[0x0070])) {
                $results[$group][] = $item[0x0070][0x0006]['data'];
            } elseif ($group != 3 && isset($item[0x0040])) {
                $tag = $item[0x0040];
                $result = $tag[0xa043][0xfffe][0xe000][0x0008][0x0104]['data'];

                if (isset($tag[0xa30a]))
                    $result .= ' ' . $tag[0xa30a]['data'];
                elseif (isset($tag[0xa132]))
                    $result .= ' ' . $tag[0xa132]['data'][0];

                if (isset($tag[0x08ea]))
                    $result .= ' ' . $tag[0x08ea][0xfffe][0xe000][0x008][0x0100]['data'];
                $results[$group][] = $result;
            }
        }
        echo safeEncode($results);

        break;

	case 'tags':

		header('Content-Type: application/json');

		$uid = HttpUtils::getValueStrict($_REQUEST, 'uid');
		$backend = getBackend(array('Structure'));
		if (!$backend->authDB->isAuthenticated())
			exit('{"error":"Not Authenticated"}');

        $depth = HttpUtils::getParam('depth', 0);
		$filters = json_decode(HttpUtils::getParam('tags', '[]'));

		$path = getPathById($uid);
		$tags = DicomTagParser::parseFile($path, $depth);

        function filter($group, $element, $filters, $tags) {
            if (isset($tags[$group]) && isset($tags[$group][$element])) {
                $tag = $tags[$group][$element];
                if (empty($filters))
                    return array(
                        'gr' => $group,
                        'el' => $element,
                        'data' => isset($tag['data']) ? $tag['data'] : $tag
                    );
                elseif (!isset($tag['data']))
                    return filter($filters[0], $filters[1], array_slice($filters, 2), $tag);
            }
            return null;
        }

		$results = array();
		if (count($filters) > 0) {
			foreach ($filters as $filter) {
				if (count($filter) % 2 !== 0)
					exit('{"error":"Tag filter has a wrong number of parameters"}');
				if (($tag = filter($filter[0], $filter[1], array_slice($filter, 2), $tags)) !== null)
					$results[] = $tag;
			}
		} else
			$results = $tags;

		$count = count($results);
		echo safeEncode($count > 0 ? ($count > 1 ? $results : $results[0]['data']) : null);

		break;

	case 'registerLicense':
		if (!empty($_REQUEST['i']))
		{
			$data = $_REQUEST;
			if (!empty($data['cmd']))
				unset($data['cmd']);
			$system = new system();
			$ansver = $system->register($data);
			$ansver = array('error' => '', 'ansver'=>$ansver);
			echo safeEncode($ansver);
		}
		else
			echo '{"error":"Missing data for licensing"}';
		exit;
		break;

	case 'activateLicense':
		$serial = HttpUtils::getValueStrict($_REQUEST, 'serial');
		$organization = HttpUtils::getValueStrict($_REQUEST, 'organization');

		$data = $_REQUEST;
		if (!empty($data['cmd']))
		{
			unset($data['cmd']);
		}
		$data['serial'] = $serial;
		$data['organization'] = $organization;
		$data['releaseDate'] = $RELDATE;
		$data['version'] = $VERSION;
		if (Constants::FOR_WORKSTATION && isset($PRODUCT))
		{
			$data['licenseFile'] = "$PRODUCT.lic";
		}
		else
		{
			$data['licenseFile'] = "meddream.lic";
		}

		$backend = getBackend(array(), false);
		$answer = $backend->gw->activateLicense($data);
		echo safeEncode($answer);
		break;

	case 'call3d':
		$seriesUid = HttpUtils::getValueStrict($_REQUEST, 'seriesUid');
		$system = new system();
		$responce = $system->call3d($seriesUid);
		$ansver = array('error' => $responce);
		echo safeEncode($ansver);
		exit;
		break;

	case 'addJob':
		if (!empty($_REQUEST['jobData']))
		{
			$data = json_decode($_REQUEST['jobData'], true);
			$jobsClass = new Jobs();
			$data = HttpUtils::getSanitizeObject($data);
			$ansver = $jobsClass->addJob($data);
			echo safeEncode($ansver);
		}
		else
			echo '{"error":"Missing jobData"}';
		exit;
		break;
    case 'getVideo':
        $uid = HttpUtils::getValueStrict($_REQUEST, 'uid');
        $path = getPathById($uid);
        unset($backend);

        $log = getLogging();
        $log->asDump("meddream_extract_meta('" . dirname(__FILE__) . "', ' " . $path . "', '0')'");
        $r = meddream_extract_meta(dirname(__FILE__), $path, 0);
        $log->asDump('meddream_extract_meta:  ', $r);

        $_REQUEST['o'] = $r['pixel_locations'][0]['offset'];
        $_REQUEST['s'] = $r['pixel_locations'][0]['size'];
        $_REQUEST['format'] = 'mp4';
        require_once 'dicom.php';
        break;

    case 'redirectToThirdPartySoftware':
		$url = HttpUtils::getValueStrict($_REQUEST, 'url', null);
        header('Location: '.$url);
        die();
        break;

	default:
		$msg = 'function not found: ' . var_export($cmd, true);
		$log = getLogging();
		$log->asErr($msg);
		HttpUtils::error('Function not found', 404);
		break;
}


/**
 * clean array from empty subarrays or null values
 *
 * @param array $data
 * @return array||string
 */
function removeEmptyArrayValues($data)
{
	if (is_null($data) || (count($data) == 0))
		return '';

	foreach ($data as $key => $value)
	{
		if (is_null($data) || (count($data) == 0))
			$value = '';
		else
			if (is_array($value))
				$value = removeEmptyArrayValues($value);
		$data[$key] = $value;
	}
	return $data;
}

function getEcg()
{
	$uid = HttpUtils::getValueStrict($_REQUEST, 'uid');
	$backend = getBackend(array('Structure'));
	if (!$backend->authDB->isAuthenticated())
		exit('{"error":"Not Authenticated"}');

	$path = getPathById($uid);
	try
	{
		$loader = new \Softneta\MedDream\Core\ECG\ECGLoader($path);
		return $loader->load(HttpUtils::getParam('filtered', 1) === 1);
	} catch (Exception $e) {
		exit('{"error":"' . $e->getMessage() . '"}');
	}
}

function safeEncode($data)
{
	$log = getLogging();

	if (version_compare(PHP_VERSION, '5.5.0', '<'))
		$flags = 0;
	else
		$flags = JSON_PARTIAL_OUTPUT_ON_ERROR;

	$j = @json_encode($data, $flags);
	$log->asDump('json_encode: ', $j, ', err: ', json_last_error());
	return $j;
}


/**
 * return new or existing Backend
 * If the underlying AuthDB must be connected to the DB, then will request the connection once more.
 *
 * @global Backend $backend
 * @param array $parts - Names of PACS parts that will be initialized
 * @param boolean $withConnection - is a DB connection required?
 * @return Backend
 */
function getBackend($parts = array(), $withConnection = true)
{
	global $backend;
	$log = getLogging();
	if (is_null($backend))
		$backend = new Backend($parts, $withConnection, getLogging());
	else
		$backend->loadParts($parts);

	if (!$backend->authDB->isConnected() && $withConnection)
		$backend->authDB->reconnect();

	return $backend;
}


/**
 * get loger class or create new
 *
 * @global Logging $log
 * @return Logging
 */
function getLogging()
{
	global $log;

	if ($log == null)
		$log = new Logging();

	return $log;
}


/**
 * change $data $search key with $change key
 * or will creates $data[$change] with $defaultValue value
 *
 * @param array $data - updates data pointer
 * @param string $search - look for $data[$search] value
 * @param string $change - will create $data[$change] = $defaultValue||$data[$search]
 * @param string|int $defaultValue - default value for non existing $change key value
 */
function changeKeyValue(&$data, $search, $change, $defaultValue)
{
	if (isset($data[$search]))
	{
		$data[$change] = $data[$search];
		unset($data[$search]);
	}
	else
		$data[$change] = $defaultValue;
}

function fixId($data)
{
	if (empty($data))
		return $data;
	$parts = explode('*', $data);
	return $parts[0];
}

function getVideoData()
{
	$log = getLogging();

	$data = json_decode(file_get_contents('php://input'), true);
	$uid = HttpUtils::getValueStrict($data, 'instanceId');

	$path = getPathById($uid);

	$log->asDump("meddream_extract_meta('" . dirname(__FILE__) . "', ' " . $path . "', '0')'");
	$r = meddream_extract_meta(dirname(__FILE__), $path, 0);
	$log->asDump('meddream_extract_meta:  ', $r);

	$result = array(
		'studyDate' => $r['studydate'],
		'seriesUID' => $r['seriesuid'],
		'accNum' => $r['accnum'],
		'ts' => $r['xfersyntax'],
		'monochrome' => 0,
		'pixelOffset' => $r['pixel_locations'][0]['offset'],
		'pixelSize' => $r['pixel_locations'][0]['size'],
		'rows' => $r['rows'],
		'columns' => $r['columns'],
		'patientName' => $r['patientname'], //todo panaikinti ^ ženklus
		'studyDateTime' => $r['studydate'] . ' ' . $r['studytime'], //todo parsintio datas
		'patientId' => $r['patientid'],
		'studyId' => $r['studyuid']
	);

	$qualityList = getVideoQuality($data["instanceId"]);
	if (empty($qualityList['error']))
	{
		$result['quality'] = $qualityList['quality'];
	}


	if (isset($r["privatevttmeta"]))
	{
		$result['privatevttmeta'] = $r["privatevttmeta"];
		$result['privatevttjpeg'] = "data:image/jpeg;base64," . base64_encode($r['privatevttjpeg']);
	}

	return safeEncode($result);
}

/**
 * return image (video) other qualities
 *
 * return json array
 * return sample:
 * {"error":"","quality":[{"quality":"Original","imageid":"id1"}
 *         ,{"quality":"1080p","imageid":"id1"},...]
 * or
 * {"error":"","quality":[]}
  or
 * {"error":"some error","quality":[]}
 */
function getVideoQuality($instanceId)
{
	$backend = getBackend(array('Structure'));
	$result = $backend->pacsStructure->collectRelatedVideoQualities($instanceId);
	unset($backend);
	return $result;
}


function buildMedDreamCmd($flags = MEDDREAM_GETRAW_JPG_STREAM)
{
    global $basedir;

    $backend = getBackend(array('Structure', 'Preload'));
    $study = new Study($backend);

    if ($backend->enableSmoothing)
        $flags |= MEDDREAM_GETRAW_ENABLE_SMOOTHING;

    $coreAddr = isset($backend->cnf->data['core_addr'])
        ? $backend->cnf->data['core_addr']
        : null;

    $log = getLogging();
    $log->asDump(__FUNCTION__ . ': $flags = ', $flags);
    return new MedDreamCmd(
        $basedir, $flags, $backend->pacsStructure, $backend->pacsPreload, $study, $coreAddr, $log,
            $backend->preparedFilesDir, $backend->preparedFilesNestingDepth);
}


function getInstanceMetadata($uid)
{
    $tags = getInstanceTags($uid);

    $log = getLogging();
    $backend = getBackend(array('Structure'));
    $dicomTags = new DicomTags($backend, $log);

    $results = new TagResults($tags, $dicomTags);

    $results->normalizeTag(array('labels', 'right', 0));
    $results->normalizeTag('patientname');

    $results->changeKey('xfersyntax', 'transferSyntaxUid');

    $results->changeKey('mssopclass', 'sopClass');
    $results->changeKey('mssopinst', 'id');
    $results->changeKey('studydate', 'studyDate');
    $results->changeKey('studytime', 'studyTime');
    $results->changeKey('accnum', 'accessionNumber');
    $results->changeKey('studydesc', 'studyDescription');
    $results->changeKey('seriesdesc', 'seriesDescription');

    $results->changeKey('patientname', 'patientName');
    $results->changeKey('patientid', 'patientId');
    $results->changeKey('patientbdate', 'patientBirthDate');
    $results->changeKey('patientsex', 'patientSex');

    $results->changeKey('slicethickness', 'sliceThickness');
    $results->changeKey('bodypart', 'bodyPartExamined');
    $results->changeKey('frametime', 'frameTime');
    $results->changeKey('rotationdirection', 'rotationDirection');

    $results->changeKey('studyuid', 'studyUid');
    $results->changeKey('seriesuid', 'seriesUid');
    $results->changeKey('sernum', 'seriesNumber');
    $results->changeKey('instancenum', 'instanceNumber');
    $results->changeKey('imageposition', 'imagePosition');
    $results->changeKey('imageorientation', 'imageOrientation');
	$results->changeKey('frameref', 'frameOfReferenceUID');

    $results->changeKey('samplesperpixel', 'samplesPerPixel');
    $results->changeKey('photometric', 'photometricInterpretation');
    $results->changeKey('numframes', 'numberOfFrames');
    $results->changeKey('pixelspacing', 'pixelSpacing');
    $results->changeKey('bitsallocated', 'bitsAllocated');
    $results->changeKey('bitsstored', 'bitsStored');
    $results->changeKey('pixelrepresentation', 'pixelRepresentation');

    $results->changeKey('windowcenter', 'windowCenter');
    $results->changeKey('windowwidth', 'windowWidth');
    $results->changeKey('rescaleintercept', 'rescaleIntercept');
    $results->changeKey('rescaleslope', 'rescaleSlope');
    $results->changeKey('rescaletype', 'rescaleType');

    $results->changeKey('imagetype', 'imageType');
    $results->changeKey('gantrytilt', 'gantryDetectorTilt');

    $results->changeKey('labels', 'viewPortLabels');
    $results->changeKey('pixelmin', 'minPixelValue');
    $results->changeKey('pixelmax', 'maxPixelValue');
    $results->changeKey('wlpixelmin', 'wlMinPixelValue');
    $results->changeKey('wlpixelmax', 'wlMaxPixelValue');
    $results->changeKey('petunits', 'units');

	$result = $results->getResults();
	//6.1.1 for reference lines to work
	if (isset($result['frameOfReferenceUID']))
	{
		$result['frameOfReferenceUid'] = $result['frameOfReferenceUID'];
	}
	if (isset($result['modality']) && $result['modality'] == 'US')
	{
		$ultrasoundRegionsData = getUltrasoundRegionsData($uid);
		if (count($ultrasoundRegionsData))
		{
			$result['ultrasoundRegionsData'] = $ultrasoundRegionsData;
		}
	}

	if (isset($result['modality']) && $result['modality'] == 'PT')
	{
	    $suvBwScaleFactor = getSuvBwScaleFactor($uid);
	    if (isset($suvBwScaleFactor))
	    {
		    $result['suvBwScaleFactor'] = getSuvBwScaleFactor($uid);
		}
	}

	return $result;
}

function getUltrasoundRegionsData($uid)
{
	$path = getPathById($uid);
	$tags = DicomTagParser::parseFile($path, 999);

	$results = array();
	if (isset($tags[0x0018][0x6011])) {
		$regionsSequence = $tags[0x0018][0x6011][0xFFFE][0xE000];
		foreach ($regionsSequence as $key => $item) {
			if (isset($item[0x0018][0x6012]['data']) && $item[0x0018][0x6012]['data'][0] == 3)  //0018,6012 Region Spatial Format == 3
			{
				$results['x0'] = $item[0x0018][0x6018]['data'][0]; //0018,6018 Region Location Min X0
				$results['y0'] = $item[0x0018][0x601A]['data'][0]; //0018,601A Region Location Min Y0
				$results['x1'] = $item[0x0018][0x601C]['data'][0]; //0018,601C Region Location Max X1
				$results['y1'] = $item[0x0018][0x601E]['data'][0]; //0018,601E Region Location Max Y1
				$results['rx0'] = $item[0x0018][0x6020]['data'][0]; //0018,6020 Reference Pixel X0
				$results['ry0'] = $item[0x0018][0x6022]['data'][0]; //0018,6022 Reference Pixel Y0
				$results['psx'] = $item[0x0018][0x602C]['data'][0]; //0018,602C Physical Delta X
				$results['psy'] = $item[0x0018][0x602E]['data'][0];  //0018,602E Physical Delta Y
			}
		}
	}
	return $results;
}


function getSuvBwScaleFactor($uid)
{
	$path = getPathById($uid);
	$tags = DicomTagParser::parseFile($path, 999);
    $suvBwScaleFactor = null;

    if (!isset($tags[0x0028][0x0051])) return 0;
    $correctedImage = $tags[0x0028][0x0051]['data'];

    if (!isset($tags[0x0054][0x1102])) return 0;
    $decayCorrection = $tags[0x0054][0x1102]['data'];

	if (strpos($correctedImage, 'ATTN') !== false && strpos($correctedImage, 'DECY') !== false && $decayCorrection == 'START') {

	    if (!isset($tags[0x0054][0x1001])) return 0;
		$units  = $tags[0x0054][0x1001]['data'];

		if ($units == 'BQML') //BQML = Becquerels/milliliter
		{
		     if (!isset($tags[0x0010][0x1030])) return 0;
             $weight  = intval($tags[0x0010][0x1030]['data']); //in kg

             if ($weight > 0)
             {
                if (!isset($tags[0x0054][0x0016]) || !isset($tags[0x0054][0x0016][0xFFFE][0xE000])) return 0;
                $ris = $tags[0x0054][0x0016][0xFFFE][0xE000]; //Radiopharmaceutical Information Sequence

                if (!isset($ris[0x0018][0x1075])) return 0;
                $rnhl = intval($ris[0x0018][0x1075]['data']); //Radionuclide Half Life

                if (!isset($tags[0x0008][0x0021])) return 0;
                $seriesDate = $tags[0x0008][0x0021]['data'];

                if (!isset($tags[0x0008][0x0031])) return 0;
                $seriesTime = $tags[0x0008][0x0031]['data'];

                if (!isset($tags[0x0008][0x0022])) return 0;
                $acquisitionDate = $tags[0x0008][0x0022]['data'];

                if (!isset($tags[0x0008][0x0032])) return 0;
                $acquisitionTime = $tags[0x0008][0x0032]['data'];

                $scanTime = null;

                if (getDateTimeFromDicomTags($seriesDate, $seriesTime) <= getDateTimeFromDicomTags($acquisitionDate, $acquisitionTime))
                {
                    $scanTime  =  getDateTimeFromDicomTags($seriesDate, $seriesTime);
                }
                else if (isset($tags[0x0009][0x0010]) && trim($tags[0x0009][0x0010]['data']) == `GEMS_PETD_01` && isset($tags[0x0009][0x100d]))
                {
                    $scanTime = getDateTimeFromDicomTag($tags[0x0009][0x100d]);
                }

                if (isset($scanTime))
                {
                    if (!isset($ris[0x0018][0x1072])) return 0;
                    $rst = $ris[0x0018][0x1072]['data']; //Radiopharmaceutical Start Time

                    if (!isset($ris[0x0018][0x1074])) return 0;
                    $injectedDose =  intval($ris[0x0018][0x1074]['data']); // Bq units

                    $measuredTime  = getDateTimeFromDicomTags($seriesDate, $rst);

                    $decayTime  =  ($scanTime - $measuredTime); //seconds
                    $decayedDose = $injectedDose * pow(2,(-$decayTime / $rnhl)); //actual activity
                    $suvBwScaleFactor = ($weight * 1000 / $decayedDose);
                }
			}
		}
		else if ($units == 'CNTS')
		{
		    if (isset($tags[0x7053][0x0010]) && trim($tags[0x7053][0x0010]['data']) == 'Philips PET Private Group' && isset($tags[0x7053][0x1000]))
		    {
                $suvBwScaleFactor = floatval($tags[0x7053][0x1000]['data']);
		    }
		    else if (isset($tags[0x7053][0x1009]))
		    {
                $suvBwScaleFactor = floatval($tags[0x7053][0x1009]['data']) * floatval($tags[0x0028][0x1053][data]);
		    }
		}
		else if ($units == 'GML')
		{
            $suvBwScaleFactor = 1.0; //assumes that GML indicates SUVbw instead of SUVlbm
        }
	}
	return $suvBwScaleFactor;
}

function getDateTimeFromDicomTags($dateTag, $timeTag)
{
	$dateTag = substr($dateTag, 0, 8);
	$dateTag = substr_replace($dateTag, '-', 4, 0);
	$dateTag = substr_replace($dateTag, '-', 7, 0);

	$timeTag = substr($timeTag, 0, 6);
	$timeTag = substr_replace($timeTag, ':', 2, 0);
	$timeTag = substr_replace($timeTag, ':', 5, 0);

	return strtotime($dateTag.' '.$timeTag);
}

function getDateTimeFromDicomTag($dateTime)
{
	$dateTag = substr(strval($dateTime), 0, 14);
	$dateTag = substr_replace($dateTag, '-', 4, 0);
	$dateTag = substr_replace($dateTag, '-', 7, 0);
	$dateTag = substr_replace($dateTag, ' ', 10, 0);
	$dateTag = substr_replace($dateTag, ':', 13, 0);
	$dateTag = substr_replace($dateTag, ':', 16, 0);
	return strtotime($dateTag);
}



function getInstanceTags($uid)
{
	$audit = new Audit('VIEW IMAGE');
	$log = getLogging();
	$backend = getBackend(array('Structure', 'Preload'));

	$flt = new ActionFilter($log);
	$flt->getActionsInto($actions);
	$err = $flt->validateAtImageLevel($uid, $actions);
	if ($err)
		HttpUtils::error($err);

	$st = $backend->pacsStructure->instanceGetMetadata($uid);
	if (strlen($st['error']))
	{
		$audit->log(false, $uid);
		return $st;
	}
	$path = $st['path'];

	$md = buildMedDreamCmd();
	$failed = false;
	try {
        $result = $md->getMeta($uid);
        $backend->pacsPreload->removeFetchedFile($path);
            // only if meddream_get_raw2 succeeded: otherwise the file is left for troubleshooting
        $audit->log(true, $uid);
    } catch (Exception $e) {
		$log->asErr('exception 1: ' . $e->getMessage());
		$failed = true;
	}

	if ($failed)
	{
		try		/* parseFile() also throws exceptions */
		{
			$tags = DicomTagParser::parseFile($path, 99);

			class TagCollector {

				private $tags;
				private $results = array();

				public function __construct($tags)
				{
					$this->tags = $tags;
				}

				public function add($gr, $el, $key) {
					if ($this->hasValue($gr, $el) && $this->hasData($gr, $el))
						$this->results[$key] = $this->tags[$gr][$el]['data'];
				}

				public function addIndexed($gr, $el, $key, $index = 0) {
					if ($this->hasValue($gr, $el) && $this->hasData($gr, $el))
						$this->results[$key] = $this->tags[$gr][$el]['data'][$index];
				}

				public function addJoined($gr, $el, $key, $delimiter) {
					if ($this->hasValue($gr, $el) && $this->hasData($gr, $el))
						$this->results[$key] = explode($delimiter, $this->tags[$gr][$el]['data']);
				}

				private function hasValue($gr, $el) {
					return isset($this->tags[$gr]) && isset($this->tags[$gr][$el]);
				}

				public function hasData($gr, $el) {
					return isset($this->tags[$gr][$el]['data']);
				}

				public function getResults()
				{
					return $this->results;
				}
			}

			$collector = new TagCollector($tags);

			$collector->add(0x0002, 0x0010, 'xfersyntax');

			$collector->add(0x0008, 0x0016, 'mssopclass');
			$collector->add(0x0008, 0x0018, 'mssopinst');
			$collector->add(0x0008, 0x0020, 'studydate');
			$collector->add(0x0008, 0x0030, 'studytime');
			$collector->add(0x0008, 0x0050, 'accnum');
			$collector->add(0x0008, 0x0060, 'modality');
			$collector->add(0x0008, 0x1030, 'studydesc');
			$collector->add(0x0008, 0x103e, 'seriesdesc');

			$collector->add(0x0010, 0x0010, 'patientname');
			$collector->add(0x0010, 0x0020, 'patientid');
			$collector->add(0x0010, 0x0030, 'patientbdate');
			$collector->add(0x0010, 0x0040, 'patientsex');

			$collector->add(0x0018, 0x0050, 'slicethickness');
			$collector->add(0x0018, 0x0015, 'bodypart');
			$collector->add(0x0018, 0x1063, 'frametime');
			$collector->add(0x0018, 0x1140, 'rotationdirection');

			$collector->add(0x0020, 0x000d, 'studyuid');
			$collector->add(0x0020, 0x000e, 'seriesuid');
			$collector->add(0x0020, 0x0011, 'sernum');
			$collector->add(0x0020, 0x0013, 'instancenum');
			$collector->addJoined(0x0020, 0x0032, 'imageposition', '\\');
			$collector->addJoined(0x0020, 0x0037, 'imageorientation', '\\');
			$collector->add(0x0020, 0x0052, 'frameref');

			$collector->addIndexed(0x0028, 0x0002, 'samplesperpixel');
			$collector->add(0x0028, 0x0004, 'photometric');
			$collector->add(0x0028, 0x0008, 'numframes');
			$collector->addIndexed(0x0028, 0x0010, 'rows');
			$collector->addIndexed(0x0028, 0x0011, 'columns');
			$collector->addJoined(0x0028, 0x0030, 'pixelspacing', '\\');
			$collector->addIndexed(0x0028, 0x0100, 'bitsallocated');
			$collector->addIndexed(0x0028, 0x0101, 'bitsstored');
			$collector->addIndexed(0x0028, 0x0103, 'pixelrepresentation');

			$collector->add(0x0028, 0x1050, 'windowcenter');
			$collector->add(0x0028, 0x1051, 'wlpixelmax');
			$collector->add(0x0028, 0x1052, 'rescaleintercept');
			$collector->add(0x0028, 0x1053, 'rescaleslope');
			$collector->add(0x0028, 0x1054, 'rescaletype');

			$result = $collector->getResults();

			$failed = false;
		}
		catch (Exception $e)
		{
			$log->asErr('exception 2: ' . $e->getMessage());
		}
	}

	if ($failed)
		$audit->log(false, $uid);

	$st = new Study($backend);
	$infoLabels = $st->getInfoLabels($uid);
	if ($infoLabels['error'] !== '')
		return $infoLabels;
	else
		$result['labels'] = $infoLabels['labels'];

	return $result;
}


function printInstancePixels($uid, $frameNum = 0)
{
    global $moduleName;

    $audit = new Audit('VIEW IMAGE');
    $log = getLogging();
    $log->asDump("begin $moduleName|" . __FUNCTION__);

    try {
        $md = buildMedDreamCmd(MEDDREAM_GETRAW_JPG_STREAM);
        $md->printRaw($uid, $frameNum);
        $audit->log(true, $uid);
    } catch (RuntimeException $e) {
        $err = $e->getMessage();
        $log->asErr('exception: ' . $err);
        $audit->log(false, $uid);
        HttpUtils::error($err);
    }

    $log->asDump("end $moduleName|" . __FUNCTION__);
}


function getAnnotationList()
{
	$studyUID = HttpUtils::getValueStrict($_POST, 'study');
	$handle = new PresentationStateHandler();
	$result = $handle->getStudyPRList($studyUID);

		//$result['error'] = 'some error'
		//$result['series'] = array(
		//						'seriesid_that_have_pr'=>
		//							array(
		//								'imageid1_that_have_pr',
		//								'imageid2_that_have_pr'..
		//							)
		//					)


	return safeEncode($result);
}


function saveAnnotationObject()
{
	$log = getLogging();
	$data = json_decode(file_get_contents('php://input'), true);
	$log->asDump('Annotation data: ', $data);
	$instanceuid = HttpUtils::getValueStrict($data, 'instanceId');
	
	/* 		$annotation = array(
	  'description'=>"description" ,
	  'title'=>'title' ,
	  'annotations'=>array(
	  array(
	  'type'=> 'TEXT',
	  'points'=>array(
	  //multiple grapgic lines
	  array('0','0','0','0'),
	  'graphicType'=> array('POLYLINE','POINT', 'CIRCLE',,..),
	  ),
	  'text'=>array(
	  array(
	  //multiple text labels or boxes
	  'text'=>'description',
	  'textpos'=> array('0','0', '10', '12'),
	  'textstyle'=>array('align'=>'LEFT')
	  )
	  )
	  )
	  )
	  );
	 */

	$log->asDump('Annotation data: ', $data);
	$handle = new PresentationStateHandler();
	$return = $handle->anotationToDicom($instanceuid, $data);

	if (isset($return['msg']))
	{
		$return['msg'] = nl2br($return['msg']);
		$log->asDump('Annotation msg: ' . $return['msg']);
	}
	if (isset($return['error']))
	{
		$return['error'] = nl2br($return['error']);
		$log->asDump('Annotation error: ' . $return['error']);
	}

	return safeEncode($return);
}


function getAnnotationObject()
{
	$log = getLogging();
	$instanceid = HttpUtils::getValueStrict($_POST, 'instanceuid');
	$handle = new PresentationStateHandler();
	$result = $handle->getImagePrlist($instanceid);

		//$result['error'] = 'some error'
		//$result['prlist'] = array(
		//						array(
		//							'description'=>"description" ,
		//							'title'=>'title' ,
		//							'type'=>'dicom',
		//							'annotations'=>array(...)
		//						),
		//						array(
		//							'description'=>"description" ,
		//							'title'=>'title' ,
		//							'type'=>'jpg',
		//							'instanceid'=>'some instance id',
		//							'path'=>'full_path_to_dicom_file'
		//						),
		//						...
		//					)
	return safeEncode($result);
}


function saveAnnotationJpeg()
{
	$log = getLogging();
	$data = json_decode(file_get_contents('php://input'), true);
	$log->asDump('Annotation data: ', $data);
	$instanceuid = HttpUtils::getValueStrict($data, 'instanceId');

	$base64img = $data['base64img'];
	$base64img = str_replace('data:image/jpeg;base64,', '', $base64img);
	$base64img = str_replace(' ', '+', $base64img);
	$pixels = base64_decode($base64img);

	$data['file'] = realpath(PATH_TO_ROOT) . DIRECTORY_SEPARATOR . 'temp' .
		DIRECTORY_SEPARATOR . 'pr' . date("YmdHis") . rand(100000, 99999999) . '.tmp';

	$tmp = $data['file'];
	file_put_contents($tmp, $pixels);
	unset($base64img);
	unset($pixels);
	unset($data['base64img']);

	$handle = new PresentationStateHandler();
	$return = $handle->jpgToDicom($instanceuid, $data);
	$handle->deleteFile($tmp);

	if (isset($return['msg']))
	{
		$return['msg'] = nl2br($return['msg']);
		$log->asDump('Annotation msg: ' . $return['msg']);
	}
	if (isset($return['error']))
	{
		$return['error'] = nl2br($return['error']);
		$log->asDump('Annotation error: ' . $return['error']);
	}

	return safeEncode($return);
}


function getPathById($imageID)
{
	$backend = getBackend(array('Structure'));
	$flt = new ActionFilter(getLogging());
	$flt->getActionsInto($actions);
	$err = $flt->validateAtImageLevel($imageID, $actions);
	if ($err)
		HttpUtils::error($err);
	$meta = $backend->pacsStructure->instanceGetMetadata($imageID);
	if ($meta['error'])
		HttpUtils::error($meta['error']);
	return $meta['path'];
}


function loadMpeg2($instanceID)
{
	require_once(PATH_TO_ROOT . 'flv.php');
	$flv = new flv(getBackend(array('Structure')));
	$result = $flv->load($instanceID, 'mp4');
	echo $result;
}


function getSettings(Logging $log, Backend $backend, System $sys)
{
	$settings = '';
	$arr = $sys->readSettingsJsonToArray();
	$arr['qr'] = array();
	$sys->addQrConfig($arr['qr']);
	$sys->addPrivilegies($arr);
	$sys->addToolLinks($arr);
	$sys->addShareConfig($arr);

	$arr['debug'] = $log->isLoggingLevelEnabled(Logging::LEVEL_DEBUG);

	if ($backend->pacs == 'DCMSYS')
	{
		if (!isset($arr['dicomView']))
			$arr['dicomView'] = array();
		$arr['dicomView']['defaultViewer'] = '0';
		if (!isset($arr['html']))
			$arr['html'] = array();
		$arr['html']['studyData'] = 2;
		$arr['html']['thumbnail'] = 2;
		$arr['html']['id'] = 'DCMSYS';
	}

	$arr = removeEmptyArrayValues($arr);

	return $arr;
}


function convertSingleElement(&$el)
{
	if (is_string($el))
		$el = htmlspecialchars($el, ENT_COMPAT | ENT_HTML401, 'UTF-8', false);
}


function breakJavascript(&$arr)
{
	array_walk_recursive($arr, 'convertSingleElement');
}


function parseReleaseNotes($path = null)
{
	$notes = array('version' => '', 'releaseDate' => '', 'notes' => array());

	/* load from default or specified path */
	if (is_null($path))
		$path = __DIR__ . '/RELNOTES';
	$lines = @file($path, FILE_SKIP_EMPTY_LINES);
	if ($lines === false)
	{
		$notes['notes'][] = array('title' => 'Error: failed to access Release Notes', 'list' => "Unreadable/missing file $path");
		return $notes;
	}

	/* group contents into the dedicated structure */
	$groupName = null;
	$groupContents = array();
	foreach ($lines as $idx => $line)
	{
		$text = trim($line);
		if (substr($text, 0, 3) == '## ')
		{
			$parts = explode(' - ', substr($text, 3));
			$notes['version'] = $parts[0];
			if (count($parts) > 1)
				$notes['releaseDate'] = $parts[1];
		}
		else
			if (substr($text, -1) == ':')
			{
				if (!is_null($groupName))
					$notes['notes'][] = array('title' => $groupName, 'list' => $groupContents);

				$groupName = substr($text, 0, -1);
				$groupContents = array();
			}
			else
				if (substr($text, 0, 2) == '- ')
					$groupContents[] = substr($text, 2);
		/* otherwise only the line "##\n" matches and we don't need it anyway */
	}
	if (!is_null($groupName))
		$notes['notes'][] = array('title' => $groupName, 'list' => $groupContents);

	return $notes;
}


function writeThumbnail($clientid, $out, $study, $s, $i, $thumbnailSize)
{
	global $backend, $constants, $basedir, $log;

	if (isset($study[$s]['modality']))
		if ($study[$s]['modality'] == 'SR')
			return '0';
	$uuid = $study[$s][$i]['id'];

	/* in some PACSes $uuid contains additional components, which are delimited by
	   characters not allowed in file names ($thumbnail below depends on $uuid);
	   let's clean them out
	 */
	$len = strpos($uuid, "*");
		if ($len !== false)
			$uuid = substr($uuid, 0, $len);

	$tempPath = $basedir."/temp/";

	global $DD;
	if ($DD)
	{

		$tempPath = session_save_path() . "/temp/";
		if (!file_exists($tempPath))
			mkdir($tempPath);
		$basedir = session_save_path() . "/";
	}

	$bitsstored = $study[$s][$i]["bitsstored"];
	$xfersyntax = $study[$s][$i]["xfersyntax"];

	if ($bitsstored == "") $bitsstored = 8;
	if ($thumbnailSize > 150)
		$thumbnailSize = 150;
	if ($thumbnailSize < 50)
		$thumbnailSize = 50;
	if ($study[$s]["count"] == 1)
		$thumbnailSize = 150;

	$thumbnail = $tempPath.$uuid.".thumbnail-".$thumbnailSize.".jpg";
	$path = str_replace("\\", "/", $study[$s][$i]['path']);

	$pathJPG = '';
	if ($constants->FDL && !Constants::DL_REGENERATE)
	{
		$pathJPG = $path.".thumbnail-".$thumbnailSize.".jpg";
		$need_new = !file_exists($pathJPG);
		if (!$need_new)
		{
			$r = file_get_contents($pathJPG);
			$log->asInfo("cached thumbnail: '$pathJPG'");
		}
	}
	else
		$need_new = true;

	if ($need_new)
	{
		if (!file_exists($thumbnail))
		{
			$fi = $backend->pacsPreload->fetchInstance($uuid, $study[$s]['id'], $study['uid']);
			if (is_string($fi))
				$path = $fi;

			if ($xfersyntax == 'jpg')
			{
				$log->asInfo("thumbnailFromJpeg('$path', $thumbnailSize)");
				$r = thumbnailFromJpeg($path, $thumbnailSize);
				if (strlen($r) == 1)		/* $r is the error location */
				{
					$log->asDump("thumbnailFromJpeg: $r");
					$r = '';
				}
			}
			else
				if (!defined('MEDDREAM_THUMBNAIL_JPG'))
				{
					$log->asErr('php_meddream does not support MEDDREAM_THUMBNAIL_JPG');
					$r = '';
				}
				else
				{
					$flags = 90 | MEDDREAM_THUMBNAIL_JPG;

					$log->asInfo("meddream_thumbnail('$path', '$thumbnail', $thumbnailSize, '$xfersyntax', $bitsstored, $flags)");
					$r =  meddream_thumbnail($path, $thumbnail, $basedir, $thumbnailSize, $xfersyntax,
						$bitsstored, Constants::THUMBNAIL_WHAT_WL, Constants::THUMBNAIL_WHAT_WL, $backend->enableSmoothing,
						$flags);
					$log->asDump("meddream_thumbnail: " . var_export(substr($r, 0, 6), true));
					if (strlen($r) > 0)
					{
						if ($r[0] == "E")			/* path to an already existing thumbnail */
						{
							$r = substr($r, 5);
							if (file_exists($r))
								$r = file_get_contents($r);
							else
								$r = "";
						}
						else
							if ($r[0] == "2")		/* GD2 data */
							{
								$r = substr($r, 5);

								if (function_exists('imagecreatefromstring'))
								{
									$r = imagecreatefromstring($r);
									ob_start();
									imagejpeg($r, NULL, 90);
									$r = ob_get_contents();
									ob_end_clean();
//									imagedestroy($r);
								}
								else
								{
									$log->asErr('GD2 extension is missing');
									$r = '';
								}
							}
							else
								if ($r[0] == 'J')
									$r = substr($r, 5);
					/* otherwise return as is; for example, it could be '?PDF' -- indication
					   to display a PDF icon
					 */
					}
					else
						$log->asErr('meddream_thumbnail failed');
				}

			/* remove the source file that might be created by fetchInstance() */
			$backend->pacsPreload->removeFetchedFile($path);
		}
		else
			$r = file_get_contents($thumbnail);
			if ($constants->FDL)
				file_put_contents($pathJPG, $r);
	}
	return $r;
}


function formJsonResult($results, $status)
{
	if (version_compare(PHP_VERSION, '5.5.0', '<'))
		$flags = 0;
	else
		$flags = JSON_PARTIAL_OUTPUT_ON_ERROR;
	$json = @json_encode($results, $flags);
	if (($json === false) || is_null($json))
	{
		$log = getLogging();
		HeaderData::setHeaderStatus(500);

		$e = 'reason unknown';
		if (function_exists('json_last_error'))
			$e = 'code ' . json_last_error();

		$log->asErr("json_encode failed ($e)");

		$json = '[{"type":"error","description":"Conversion to JSON failed"}]';
	}
	else
		HeaderData::setHeaderStatus($status);

	HeaderData::setHeaderJson();
	return $json;
}


function updateWithLiveData(&$data, $searchCriteria)
{
	$log = getLogging();
	$log->asInfo('updating search result with live stream data from liveStreamList.json');

	$streamListPath = __DIR__ . DIRECTORY_SEPARATOR . 'liveStreamList.json';
	if (!file_exists($streamListPath))
	{
		$log->asWarn('file does not exist: ' . $streamListPath);
		return;
	}
	$jsonData = @file_get_contents($streamListPath);
	if ($jsonData === false)
	{
		$log->asErr('failed to read file: ' . $streamListPath);
		return;
	}
	$log->asDump('JSON data from stream list: ', $jsonData);
	$arrayData = @json_decode($jsonData, true);
	if (is_null($arrayData))
	{
		$log->asErr('failed to parse file: ' . $streamListPath);
		return;
	}
	$log->asDump('parsed stream list: ', $arrayData);
	if (empty($arrayData['liveStream']))
	{
		$log->asWarn('no liveStream found in: ' . $streamListPath);
		return;
	}
	$items = $arrayData['liveStream'];
	$count = count($items);
	$log->asInfo('found live streams: '. $count);

	if ($count == 0)
		return;

	$id = '';
	$name = '';
	$description = '';
	$sourceAe = '';
	$crCount = count($searchCriteria);
	for ($i = 0; $i < $crCount; $i++)
	{
		if (!array_key_exists('name', $searchCriteria[$i]) ||
				!array_key_exists('text', $searchCriteria[$i]))
			continue;
		if ($searchCriteria[$i]['name'] == 'patientid')
			$id = strtolower($searchCriteria[$i]['text']);
		else
			if ($searchCriteria[$i]['name'] == 'patientname')
				$name = strtolower($searchCriteria[$i]['text']);
		else
			if ($searchCriteria[$i]['name'] == 'description')
				$description = strtolower($searchCriteria[$i]['text']);
		else
			if ($searchCriteria[$i]['name'] == 'sourceae')
				$sourceAe = strtolower($searchCriteria[$i]['text']);
	}
	$log->asDump('$searchCriteria: ' , $searchCriteria);
	$log->asDump('searching in live stream: id ', $id, ', name ', $name, ', description, ', $description, ', sourceAE ', $sourceAe);

	for ($i = 0; $i < $count; $i++)
	{
		$item = array();
		if (array_key_exists('id', $items[$i]))
		{
			$item['uid'] = $items[$i]['id']; // unknown
			if (($id != '') &&
					(strpos(strtolower($items[$i]['id']), $id) === false))
				continue;
			$item['id'] = $items[$i]['id'];
		}
		else
			continue;

		if (array_key_exists('name', $items[$i]))
		{
			if (($name != '') &&
					(strpos(strtolower($items[$i]['name']), $name) === false))
				continue;
			$item['name'] = $items[$i]['name'];
		}

		if (array_key_exists('modality', $items[$i]))
			$item['modality'] = $items[$i]['modality'];
		else
			$item['modality'] = 'LIVE'; //just in case

		if (array_key_exists('description', $items[$i]))
		{
			if (($description != '') &&
					(strpos(strtolower($items[$i]['description']), $description) === false))
				continue;
			$item['description'] = $items[$i]['description'];
		}
		if (array_key_exists('sourceAE', $items[$i]))
		{
			if (($sourceAe != '') &&
					(strpos(strtolower($items[$i]['sourceAE']), $sourceAe) === false))
				continue;
			$item['sourceae'] = $items[$i]['sourceAE'];
		}
		$item['isRead'] = false;
		$item['haveReport'] = false;
		$item['availableReport'] = false;
		$item['haveAnnotation'] = false;

		$item['name'] = trim(preg_replace('/\s+/', ' ', $item['name']));
		$item['modality'] = trim(preg_replace('/\s+/', ' ', $item['modality']));
		$item['description'] = trim(preg_replace('/\s+/', ' ', $item['description']));
		$data[] = $item;
	}
}