<?php
/*
	Move this file into MedDream installation directory in order to use
 */
use Softneta\MedDream\Core\Database\DB;

define('PASSWORD_SALT', '412795636');

define('DCM4CHE_DIR', __DIR__ . '/dcm4che/bin');

define('DUMP_SIZE', 2048);
define('DUMP_COLUMNS', 4);


class DamageAnalysis
{
	private $f;
	private $isExplicit;
	private $prevGroup;

	private function readRaw($size, $eofExpected = false)
	{
		$raw = fread($this->f, $size);
		$found = strlen($raw);
		if ($found != $size)
		{
			if ($eofExpected)
				if (!$found)
					return null;
				else
					return $raw;
			echo "!!! read $found byte(s) instead of $size\n";
			return null;
		}
		return $raw;
	}


	private function skipBytes($count)
	{
		if (fseek($this->f, $count, SEEK_CUR))
		{
			echo "!!! seek failed (relative offset $count)\n";
			return false;
		}
		return true;
	}


	/* false: file pointer wasn't moved
	   true: file pointer is at beginning of next tag
	   null: EOF
	 */
	private function rubbishFoundAndSkipped($chunkOf4Bytes, $isInside)
	{
		if (!preg_match('/^(Mon|Tue|Wed|Thu|Fri|Sat|Sun) $/', $chunkOf4Bytes))
			return false;

		$garbageTotal = 4;

		/* if the text replaces part of VR/VL, then it becomes rather machine-unreadable.
		   Let's make some guesses based on known situations and context.
		 */
		if ($isInside)
		{
			$foundEol = 0;

			for (;;)
			{
				$bin = $this->readRaw(128, true);	/* 128: a bit more than the typical length */
				if (is_null($bin))
				{
					echo "!!! PacsOne logs (1) #$garbageTotal\n";
					return null;
				}

				for ($i = 0; $i < strlen($bin); $i++)
				{
					if ($foundEol == 3)
					{
						if (preg_match('/[0-9\. ]/', $bin[$i]))
							continue;

						$rollbackSize = strlen($bin) - $i;
						if (!$this->skipBytes(-$rollbackSize))
							return null;

						echo '!!! PacsOne logs (2) #' . ($garbageTotal + $i) . "\n";
						return true;
					}
					else
						if (preg_match('/[0-9A-Za-z<>, :\.\-\[\]]/', $bin[$i]))
							$foundEol = 0;
						else
						{
							if (($bin[$i] == "\r") && !$foundEol)
							{
								$foundEol = 1;
								continue;
							}
							if (($bin[$i] == "\n") && ($foundEol == 1))
							{
								$foundEol = 2;
								continue;
							}
							if (($bin[$i] == "\0") && ($foundEol == 2))
							{
								$foundEol = 3;
								continue;
							}

							echo '??? unknown character[' . ord($bin[$i]) . '] at ' . ($garbageTotal + $i) . " in PacsOne logs (1)\n";
							return null;
						}
				}

				$garbageTotal += strlen($bin);
			}
		}
		else
		{
			$foundEol = 0;

			for (;;)
			{
				$bin = $this->readRaw(128, true);	/* 128: a bit more than the typical length */
				if (is_null($bin))
				{
					echo "!!! PacsOne logs (3) #$garbageTotal\n";
					return null;
				}

				for ($i = 0; $i < strlen($bin); $i++)
				{
					if (preg_match('/[0-9A-Za-z<> :,\.\-\[\]]/', $bin[$i]))
						$foundEol = 0;
					else
					{
						if (($bin[$i] == "\r") && !$foundEol)
						{
							$foundEol = 1;
							continue;
						}
						if (($bin[$i] == "\n") && ($foundEol == 1))
						{
							$foundEol = 2;
							continue;
						}
						$grp = substr($bin, $i, 2);
						if (($grp[1] === $this->prevGroup[1]) && ($grp[0] >= $this->prevGroup[0]) && ($foundEol == 2))
						{
							$rollbackSize = strlen($bin) - $i;
							if (!$this->skipBytes(-$rollbackSize))
								return null;

							echo '!!! PacsOne logs (4) #' . ($garbageTotal + $i) . "\n";
							return true;
						}

						echo '??? unknown character[' . ord($bin[$i]) . '] at ' . ($garbageTotal + $i) . " in PacsOne logs (2)\n";
						return null;
					}
				}

				$garbageTotal += strlen($bin);
			}
		}

		return true;
	}


	private function readTagHdr()
	{
		$bin = $this->readRaw(4, true);
		if (is_null($bin))
			return null;
		$found = $this->rubbishFoundAndSkipped($bin, false);
		if (is_null($found))
			return null;
		if ($found)
			return false;
		$ge = unpack('v2', $bin);
		$grp = $ge[1];
		$this->prevGroup = substr($bin, 0, 2);
		$elm = $ge[2];
		$vr = '';

		if ((($grp == 0x0002) || $this->isExplicit) && (($grp != 0xFFFE) && ($elm != 0xE000)))
		{
			$bin = $this->readRaw(4);
			$found = $this->rubbishFoundAndSkipped($bin, true);
			if (is_null($found))
				return null;
			if ($found)
				return false;
			$vl = unpack('c2vr/vlen', $bin);
			$vr = chr($vl['vr1']) . chr($vl['vr2']);
			$reservedOrLength = $vl['len'];
			if (($vr == 'OB') || ($vr == 'OW') || ($vr == 'OF') || ($vr == 'SQ') || ($vr == 'UT') || ($vr == 'UN'))
			{
				$bin = $this->readRaw(4);
				$vl = unpack('V', $bin);
				$len = $vl[1];
			}
			else
				$len = $reservedOrLength;
		}
		else
		{
			$bin = $this->readRaw(4);
			$found = $this->rubbishFoundAndSkipped($bin, true);
			if (is_null($found))
				return null;
			if ($found)
				return false;
			$vl = unpack('V', $bin);
			$len = $vl[1];
		}

		return array($grp, $elm, $vr, $len);
	}


	private function readTag($level)
	{
		$pos = ftell($this->f);

		$h = $this->readTagHdr();
		if (is_null($h))
			return null;
		if ($h === false)
			return false;
		list($grp, $elm, $vr, $len) = $h;
		$vrStr = strlen($vr) ? "$vr " : '';

		echo sprintf('%d:', $pos);
		echo str_repeat('>', $level);
		echo sprintf("(%04X,%04X) %s#%d\n", $grp, $elm, $vrStr, $len);

		if (($grp == 0x0002) && ($elm == 0x0010))
		{
			$uid = trim($this->readRaw($len));
			$this->isExplicit = $uid != '1.2.840.10008.1.2';
			return false;
		}

		if (($grp == 0xFFFE) && (($elm == 0xE00D) || ($elm == 0xE0DD)))
			return true;

		if ($len == -1)
		{
			do
			{
				$ended = $this->readTag($level + 1);
				if (is_null($ended))
					return null;
			} while (!$ended);
		}
		else
			if (!$this->skipBytes($len))
				return null;

		return false;
	}


	public function exam($in)
	{
		clearstatcache(false, $in);
		if (!is_file($in))
		{
			echo "!!! missing file: $in\n";
			return;
		}

		$this->isExplicit = false;
		$this->prevGroup = '';
		$this->f = fopen($in, 'rb');
		if (!$this->f)
			return;

		$bin = $this->readRaw(4);
		if ($bin != 'DICM')
		{
			$this->skipBytes(124);
			$bin = $this->readRaw(4);
			if ($bin != 'DICM')
			{
				fclose($this->f);
				echo "!!! no format marker found: $in\n";
				return;
			}
		}

		do
		{
			$read = $this->readTag(0);
		} while (!is_null($read));

		fclose($this->f);
		$this->f = null;
	}
}


function get_DB()
{
	require_once(PORTIONS_DIR . '/autoload.php');

	$db = new DB('MySQL', DB_HOST, DB_NAME, DB_USER, DB_PASSWORD);
	$err = $db->reconnect();
	return array($db, $err);
}


function print_dump($filename)
{
	/* read up to DUMP_BYTES from the file */
	@trigger_error('');
	$fp = @fopen($filename, 'rb');
	if (!$fp)
	{
		$err = error_get_last();

		if ($err)
			echo $err['message'];
		else
			echo 'failed to open';

		echo "\n";
		return;
	}
	$chunk = @fread($fp, DUMP_SIZE);
	fclose($fp);

	/* display line by line */
	$ofs = 0;
	while ($ofs < strlen($chunk))
	{
		echo sprintf('%04X:', $ofs);

		/* as many columns of codes as needed */
		for ($j = 0; $j < DUMP_COLUMNS; $j++)
		{
			$len = strlen($chunk) - $ofs - ($j * 8);
			if ($len < 0)
				$len = 0;
			else
				if ($len > 8)
					$len = 8;

			for ($k = 0; $k < $len; $k++)
			{
				$chr = $chunk[$ofs + ($j * 8) + $k];
				echo ' ' . sprintf('%02X', ord($chr));
			}
			for ($k = $len; $k < 8; $k++)
				echo '   ';

			echo ' ';
			if (($j + 1) < DUMP_COLUMNS)
				echo '|';
		}

		/* add a corresponding amount of raw characters */
		$len = strlen($chunk) - $ofs;
		if ($len > (DUMP_COLUMNS * 8))
			$len = DUMP_COLUMNS * 8;

		for ($k = 0; $k < $len; $k++)
		{
			if (!($k % 8))
				echo ' ';

			$chr = $chunk[$ofs + $k];

			/* adjust values for proper display. The output shall be in Latin1 enforced by header(). */
			$val = ord($chr);
			if ((($val >= 0) && ($val < 32)) ||		/* nonprintable */
				(($val > 126) && ($val < 0xA0)))	/* undefined in Latin1 */
				$chr = '.';
			else
				if ($val == 0xAD)					/* soft hyphen, invisible */
					$chr = '-';
				else
					if ($chr == '>')
						$chr = '&gt;';
					else
						if ($chr == '<')
							$chr = '&lt;';
			echo $chr;
		}

		echo "\n";
		$ofs += $len;
	}
}


function size_str($size)
{
	$val = $size;
	$suf = 'B';
	$fmt = '%d';

	if ($val > 1024)
	{
		$val /= 1024;
		$suf = 'KB';
		$fmt = '%.1f';
	}
	if ($val > 1024)
	{
		$val /= 1024;
		$suf = 'MB';
	}
	if ($val > 1024)
	{
		$val /= 1024;
		$suf = 'GB';
	}
	if ($val === false)
		return '??? ' . $suf;
	else
		return sprintf($fmt, $val) . ' ' . $suf;
}


function get_worker_status($proc)
{
	$file = STATUS_FILE_PREFIX . "$proc.index";
	$current = @file_get_contents($file);
	if ($current === false)
		$current = '?';
	return $current;
}


error_reporting(E_ALL | E_STRICT);
clearstatcache();
$prep_dir = 'scripts/prep/';

$tm_script_started = time();
//if (date('I') == '0')
//	$tm_script_started += 3600;

$cfg0 = $prep_dir . 'config.php';
$cfg1 = dirname(__FILE__) . DIRECTORY_SEPARATOR . $cfg0;
if (!@file_exists($cfg1))
	exit('not found: ' . $cfg0);
include($cfg1);

/* a cookie login: preserve the current value in the cookie */
if (isset($_POST['op']) && ($_POST['op'] == 'login'))
{
	$dom = $_SERVER['SERVER_NAME'];
	if (($dom == 'localhost') || ($dom == '127.0.0.1'))
		$dom = '';
	$h = md5(PASSWORD_SALT . strrev($_SERVER['REMOTE_ADDR']) . $_POST['password']);
	setcookie('prepared_password', $h, time() + 3600 * 24 * 7, '/', $dom);
	header('Location: ' . basename(__FILE__));
	exit;
}

/* a cookie login: verification */
$logged_in = 1;
if (!isset($_COOKIE['prepared_password']))
	$logged_in = 0;
else
{
	if (!strlen(WEB_PASSWORD) || (WEB_PASSWORD == '`UPguRW3/s'))	/* the latter value is (in a sense) compromised */
		exit('Please set up a non-empty and non-default WEB_PASSWORD in ' . $cfg0);
	$h = md5(PASSWORD_SALT . strrev($_SERVER['REMOTE_ADDR']) . WEB_PASSWORD);
	if ($_COOKIE['prepared_password'] != $h)
		$logged_in = -1;
}
if ($logged_in != 1)
{
	/* reset the current value to avoid endless redirection */
	set_time_limit(0);		/* less CPU load in usleep() */
	if ($logged_in < 0)
	{
		usleep(1000 * rand(1000, 3000));
		$dom = $_SERVER['SERVER_NAME'];
		if (($dom == 'localhost') || ($dom == '127.0.0.1'))
			$dom = '';
		setcookie('prepared_password', '', time() - 3600 * 24 * 365, '/', $dom);
	}
?><!DOCTYPE html>
<html><body>
	<form name="login" method="POST" action="<?php echo basename(__FILE__); ?>">
Password:
		<input type="password" name="password">
		<input type="hidden" name="op" value="login">
		<input type="submit" name="submit" value="Login">
	</form>
</body></html>
<?php
	exit;
}

if (isset($_REQUEST['op']))
{
	$out_suf = "</pre></body></html>\n";

	switch ($_REQUEST['op'])
	{
	case 'log':
		if (isset($_REQUEST['day']))
		{
			$day = $_REQUEST['day'];
			if ($day === '.')
				$day = -1;		/* a different kind of log */
		}
		else
			$day = 0;
		if ($day >= 0)
		{
			if (!isset($_REQUEST['wrk']))
				exit('wrong parameters');
			else
				$proc = $_REQUEST['wrk'];
		}
		else
			$proc = 0;			/* not valid, however we'll get a clearer error message when using it */

		$out_pre = '<html>';
		if (!$day)		/* current log */
			$out_pre .= '<head><meta http-equiv="refresh" content="300"></head>';
		else			/* older logs and free space log */
			$out_pre .= '<head><meta http-equiv="refresh" content="21600"></head>';
		if (!$day)
			$out_pre .= '<body onload="javascript:window.scrollTo(0, document.body.scrollHeight);">';
		$out_pre .= "<pre>\n";

		if ($day < 0)
			$log0 = $prep_dir . DIRECTORY_SEPARATOR . 'diskfree.log';
		else
		{
			$tm = time() - 3600 * 24 * $day;
			$log0 = $prep_dir . DIRECTORY_SEPARATOR . "prep$proc-" . date('Ymd', $tm) . '.log';
		}
		$log1 = dirname(__FILE__) . DIRECTORY_SEPARATOR . $log0;

		$size = @filesize($log1);
		if ($size === false)
			exit('not found: ' . $log0);
		$size1 = strlen($out_pre) + $size + strlen($out_suf);

		header('Content-Type: text/html');
		header("Content-Length: $size1");

		echo $out_pre;

		$f = fopen($log1, 'rb');
		while ($size && !connection_aborted())
		{
			$bs = 32768;
			if ($size < $bs)
				$bs = $size;

			print fread($f, $bs);
			flush();

			$size -= $bs;
		}
		fclose($f);

		echo $out_suf;
		flush();

		break;

	case 'dump':
		if (!isset($_REQUEST['uid']) || !strlen($_REQUEST['uid']))
			exit('wrong parameters');
		$uid = htmlspecialchars($_REQUEST['uid']);

		/* preserve &all=1 */
		$query_prm_unlim = '';
		if (isset($_REQUEST['all']))
			if (1 == $_REQUEST['all'])
				$query_prm_unlim = '&all=1';

		header('Content-Type: text/html; charset=ISO-8859-1');
		echo "<html><body>";
		echo "\n";
		echo '<a href="' . basename(__FILE__) . '?op=download' . $query_prm_unlim;
		echo '&uid=' . $uid . '">[&nbsp;Download&nbsp;]</a>';
		echo ' &nbsp; ';
		echo '<a href="' . basename(__FILE__) . '?op=export' . $query_prm_unlim;
		echo '&uid=' . $uid . '">[&nbsp;Export&nbsp;]</a>';
		echo ' &nbsp; ';
		echo '<a href="' . basename(__FILE__) . '?op=retry' . $query_prm_unlim;
		echo '&uid=' . $uid . '">[&nbsp;Retry&nbsp;]</a>';
		echo ' &nbsp; ';
		echo '<a href="' . basename(__FILE__) . '?op=kill' . $query_prm_unlim;
		echo '&uid=' . $uid . '">[&nbsp;Remove&nbsp;]</a>';
		echo "\n";
		echo "<pre>\n";

		echo "<b>Record</b>\n\n";

		$result = $file = NULL;
		list($db, $err) = get_DB();
		if (!$err)
		{
			$uid_safe = $db->sqlEscapeString($uid);
			if ($uid_safe === false)
				$result = 'sqlEscapeString: ' . $db->getError();
			else
			{
				$rs = $db->query('SELECT path FROM thn WHERE ' . F_IMAGE_UUID . "='$uid_safe'");
				if (!$rs)
					$result = 'query: ' . $db->getError();
				else
				{
					$row = $db->fetchNum($rs);
					if (!$row)
						echo 'fetchNum: record not found for ' . var_export($uid_safe, true);
					else
					{
						$file = $row[0];
						if (PHP_OS == 'WINNT')
							$file = str_replace('/', '\\', $file);
					}
				}
			}
			if (!is_null($result))
			{
				$db->close();
				$result = "<span style='color: red; font-weight: bold'>$result</span>\n";
				exit($result);
			}

			$rs = $db->query('SELECT COUNT(*) FROM image WHERE ' . F_IMAGE_UUID . "='$uid_safe'");
			if (!$rs)
				$result = 'query: ' . $db->getError();
			else
			{
				$row = $db->fetchNum($rs);
				$result = $row[0] ? 'YES' : 'NO';
			}
			$db->close();
		}
		else
			$result = $err;

		echo 'Also exists in `image` table? ';
		if ($result != 'YES')
			$result = "<span style='color: red; font-weight: bold'>$result</span>\n";
		echo "$result\n<p>";

		echo "<b>File</b>\n\n";
		echo "Path:        $file\n";
		$st = @stat($file);
		if ($st === false)
			echo "<span style='color: red; font-weight: bold'>not accessible</span>\n";
		else
		{
			echo "Size, bytes: " . $st['size'] . "\n";
			echo "Created:     " . date('Y-m-d H:i:s', $st['ctime']) . "\n";
			echo "Modified:    " . date('Y-m-d H:i:s', $st['mtime']) . "\n";
		}

		echo "\n<b>Damage analysis</b>\n\n";
		$da = new DamageAnalysis();
		ob_start();
		$da->exam($file);
		$buf = ob_get_clean();
		$analysis = '';
		foreach (explode("\n", $buf) as $line)
		{
			$first = substr($line, 0, 3);
			if (($first === '!!!') || ($first === '???'))
				$analysis .= "<b><font color=\"#ff0000\">$line</font></b>\n";
			else
				$analysis .= "<font color=\"#bfbfbf\">$line</font>\n";
		}
		echo $analysis;

		echo "\n<b>Raw dump</b>\n\n";
		print_dump($file);

		echo "\n<b>Tag dump</b>\n\n";
		set_time_limit(0);
		flush();
		$tmp = tempnam(dirname(__FILE__) . DIRECTORY_SEPARATOR . 'tmp', 'd2t');

		if (PHP_OS == 'WINNT')
			$exe = 'CMD /C ""' . DCM4CHE_DIR . '\\dcm2txt.bat" -w 200 -l 100 "' . $file . '" >"' . $tmp . '"" 2>&1';
		else
			$exe = DCM4CHE_DIR . '/dcm2txt -w 200 -l 100 "' . $file . '" >"' . $tmp .'" 2>&1';
		$err = '';
		$errors = '';
		try
		{
			exec($exe, $err);
		}
		catch (Exception $e)
		{
			$errors = $e->getMessage();
		}
		if (empty($errors))
			$errors = implode("\n", $err);
		if (!empty($errors))
			echo "dcm2txt failed: '" . $errors . "'";

		$f = fopen($tmp, 'rb');
		while (!feof($f) && !connection_aborted())
		{
			$chunk = fread($f, 32768);
			$chunk = str_replace('<', '&lt;', $chunk);
			print $chunk;
			flush();
		}
		fclose($f);
		unlink($tmp);

		echo $out_suf;
		flush();
		break;

	case 'kill':
		if (!isset($_REQUEST['uid']) || !strlen($_REQUEST['uid']))
			exit('wrong parameters');
		$uid_list = explode(',', htmlspecialchars($_REQUEST['uid']));

		list($db, $err) = get_DB();
		if (!$err)
		{
			/* preserve &all=1 */
			$unlim_prm = '';
			if (isset($_REQUEST['all']))
				if (1 == $_REQUEST['all'])
					$unlim_prm = '?all=1';

			$uids_safe = array();
			foreach ($uid_list as $uid)
			{
				$uid_safe = $db->sqlEscapeString($uid);
				if ($uid_safe === false)
					exit('sqlEscapeString: ' . $db->getError());
				$uids_safe[] = $uid_safe;
			}
			$uids = implode("', '", $uids_safe);

			$rs = $db->query('DELETE FROM thn WHERE ' . F_IMAGE_UUID . " IN ('$uids')");
			if (!$rs)
				echo 'query: ' . $db->getError();
			else
				header('Location: ' . basename(__FILE__) . $unlim_prm);

			$db->close();
		}
		else
			echo($err);
		break;

	case 'retry':
		if (!isset($_REQUEST['uid']) || !strlen($_REQUEST['uid']))
			exit('wrong parameters');
		$uid_list = explode(',', htmlspecialchars($_REQUEST['uid']));

		list($db, $err) = get_DB();
		if (!$err)
		{
			/* preserve &all=1 */
			$unlim_prm = '';
			if (isset($_REQUEST['all']))
				if (1 == $_REQUEST['all'])
					$unlim_prm = '?all=1';

			$uids_safe = array();
			foreach ($uid_list as $uid)
			{
				$uid_safe = $db->sqlEscapeString($uid);
				if ($uid_safe === false)
					exit('sqlEscapeString: ' . $db->getError());
				$uids_safe[] = $uid_safe;
			}
			$uids = implode("', '", $uids_safe);

			/* set date in the past, and detach from current worker so that
			   the first idle one could take the job ASAP
			 */
			$rs = $db->query("UPDATE thn SET worker=0,nexttryat=DATE_SUB(NOW(), INTERVAL 1 SECOND),error='' WHERE " .
				F_IMAGE_UUID . " IN('$uids')");
			if (!$rs)
				echo 'query: ' . $db->getError();
			else
				header('Location: ' . basename(__FILE__) . $unlim_prm);

			$db->close();
		}
		else
			echo($err);
		break;

	case 'retryall':
		list($db, $err) = get_DB();
		if (!$err)
		{
			/* preserve &all=1 */
			$unlim_prm = '';
			if (isset($_REQUEST['all']))
				if (1 == $_REQUEST['all'])
					$unlim_prm = '?all=1';

			/* set date in the past, and detach from current worker so that
			   the first idle one could take the job ASAP
			 */
			$rs = $db->query("UPDATE thn SET worker=0,nexttryat=DATE_SUB(NOW(), INTERVAL 1 SECOND),error='' WHERE numoftries<>0");
			if (!$rs)
				echo 'query: ' . $db->getError();
			else
				header('Location: ' . basename(__FILE__));

			$db->close();
		}
		else
			echo($err);
		break;

	case 'export':
		if (!isset($_REQUEST['uid']) || !strlen($_REQUEST['uid']))
			exit('wrong parameters');
		$uid_list = explode(',', htmlspecialchars($_REQUEST['uid']));

		list($db, $err) = get_DB();
		if (!$err)
		{
			$uids_safe = array();
			foreach ($uid_list as $uid)
			{
				$uid_safe = $db->sqlEscapeString($uid);
				if ($uid_safe === false)
					exit('sqlEscapeString: ' . $db->getError());
				$uids_safe[] = $uid_safe;
			}
			$uids = implode("', '", $uids_safe);

			$rs = $db->query('SELECT uuid, path, xfersyntax, bitsstored, error, numoftries FROM thn WHERE ' .
				F_IMAGE_UUID . " IN ('$uids')");
			if (!$rs)
				echo 'query: ' . $db->getError();
			else
			{
				echo "<html><body><pre>\n";
				while ($row = $db->fetchAssoc($rs))
					echo "'" . $row['uuid'] . "';'" . $row['xfersyntax'] . "';" . $row['bitsstored'] .
						";'" . $row['error'] . "';" . $row['numoftries'] . ";'" . $row['path'] .
						"'\n";
				echo "</pre></body></html>\n";
			}

			$db->close();
		}
		else
			echo($err);
		break;

	case 'download':
		if (!isset($_REQUEST['uid']) || !strlen($_REQUEST['uid']))
			exit('wrong parameters');
		$uid = htmlspecialchars($_REQUEST['uid']);

		list($db, $err) = get_DB();
		if (!$err)
		{
			$uid_safe = $db->sqlEscapeString($uid);
			if ($uid_safe === false)
				echo 'sqlEscapeString: ' . $db->getError();
			else
			{
				$rs = $db->query('SELECT path FROM thn WHERE ' . F_IMAGE_UUID . "='$uid_safe'");
				if (!$rs)
					echo 'query: ' . $db->getError();
				else
				{
					$row = $db->fetchNum($rs);
					if (!$row)
						echo 'fetchNum: record not found for ' . var_export($uid_safe, true);
					else
					{
						$path = $row[0];
						$fh = fopen($path, 'rb');
						if ($fh)
						{
								header('Content-Type: application/octet-stream');
							header('Content-Disposition: attachment; filename="' . basename($path) .
								'.dcm"');

							$prop = @fstat($fh);
							if ($prop)
								header('Content-Length: ' . $prop['size']);

							fpassthru($fh);
							fclose($fh);
						}
					}
				}
			}

			$db->close();
		}
		else
			echo($err);
		break;

	case 'logoff':
		$dom = $_SERVER['SERVER_NAME'];
		if (($dom == 'localhost') || ($dom == '127.0.0.1'))
			$dom = '';
		setcookie('prepared_password', '', time() - 3600 * 24 * 365, '/', $dom);
		header('Location: ' . basename(__FILE__));
	}
	exit;
}

$prep0 = $prep_dir . DIRECTORY_SEPARATOR . 'prepare.php';
$prep1 = dirname(__FILE__) . DIRECTORY_SEPARATOR . $prep0;
if (!@file_exists($prep1))
	exit('not found: ' . $prep0);
include($prep1);
?>
<!DOCTYPE html>
<html><head><meta http-equiv="refresh" content="1020">
<script language="JavaScript">
function groupSelection(value)
{
	var inputs = document.getElementsByName("uuid");
	for (var i = 0; i < inputs.length; i++)
		inputs[i].checked = value;
}

function groupAction(action, is_unlimited)
{
	var inputs = document.getElementsByName("uuid");
	var uids = new Array();
	for (var i = 0; i < inputs.length; i++)
		if (inputs[i].checked)
			uids.push(inputs[i].value);
	var prm = uids.join();
	if (!prm.length)
		alert('Nothing is selected');
	else
	{
		var field = document.getElementById("groupInput");
		field.setAttribute("value", prm);
		field = document.getElementById("groupOp");
		field.setAttribute("value", action);
		field = document.getElementById("passUnlimited");
		field.setAttribute("value", is_unlimited);

		var form = document.forms[0];
		form.submit();
	}
}
</script>
</head><body>
<table width="100%" border="0">
	<tr>
		<td align="left"><?php echo date('Y-m-d H:i:s', $tm_script_started); ?></td>
		<td align="right"><a href="<?php echo basename(__FILE__); ?>?op=logoff">[&nbsp;Logoff&nbsp;]</a></td>
	</tr>
</table>
<?php

/* display the state of the directory for prepared files */
$df = disk_free_space(DST_DIR);
if ($df === false)
	$df_mb = -1;
else
	$df_mb = floor($df / 1024 / 1024);
$is_stuck = $df_mb < REQD_FREE_MB_LO;
echo "<b>Directory '" . DST_DIR . "': ";
if ($is_stuck)
	echo '<span style="color: red">';
echo "$df_mb MB free, " . REQD_FREE_MB_LO . ' MB required';
if ($is_stuck)
	echo '</span>';
echo "</b>\n";

/* collect worker status */
$worker_status = array();
//'11', '22', '33', '44', '55', '66');
if (!NUM_OF_COPIES)
	$worker_status[] = get_worker_status(0);
else
	for ($i = 0; $i < NUM_OF_COPIES; $i++)
		$worker_status[] = get_worker_status($i + 1);

/* determine sizes of log files */
$logsize0 = array();
for ($i = 0; $i < NUM_OF_COPIES; $i++)
{
	$proc = $i + 1;
	$log0 = $prep_dir . DIRECTORY_SEPARATOR . "prep$proc-" . date('Ymd', $tm_script_started) . '.log';
	$log1 = dirname(__FILE__) . DIRECTORY_SEPARATOR . $log0;
	$logsize0[] = size_str(@filesize($log1));
}
/* workers 0 and 99 */
$log0 = $prep_dir . DIRECTORY_SEPARATOR . "prep0-" . date('Ymd', $tm_script_started) . '.log';
$log1 = dirname(__FILE__) . DIRECTORY_SEPARATOR . $log0;
$logsize0[] = size_str(@filesize($log1));
if (NUM_OF_COPIES)
{
	$log0 = $prep_dir . DIRECTORY_SEPARATOR . "prep99-" . date('Ymd', $tm_script_started) . '.log';
	$log1 = dirname(__FILE__) . DIRECTORY_SEPARATOR . $log0;
	$logsize0[] = size_str(@filesize($log1));
}

$tm = $tm_script_started - 3600 * 24;
$logsize1 = array();
for ($i = 0; $i < NUM_OF_COPIES; $i++)
{
	$proc = $i + 1;
	$log0 = $prep_dir . DIRECTORY_SEPARATOR . "prep$proc-" . date('Ymd', $tm) . '.log';
	$log1 = dirname(__FILE__) . DIRECTORY_SEPARATOR . $log0;
	$logsize1[] = size_str(@filesize($log1));
}
$log0 = $prep_dir . DIRECTORY_SEPARATOR . "prep0-" . date('Ymd', $tm) . '.log';
$log1 = dirname(__FILE__) . DIRECTORY_SEPARATOR . $log0;
$logsize1[] = size_str(@filesize($log1));
if (NUM_OF_COPIES)
{
	$log0 = $prep_dir . DIRECTORY_SEPARATOR . "prep99-" . date('Ymd', $tm) . '.log';
	$log1 = dirname(__FILE__) . DIRECTORY_SEPARATOR . $log0;
	$logsize1[] = size_str(@filesize($log1));
}

$log0 = $prep_dir . DIRECTORY_SEPARATOR . 'diskfree.log';
$log1 = dirname(__FILE__) . DIRECTORY_SEPARATOR . $log0;
$logsize_ = size_str(@filesize($log1));
?>
<br>
<br>
<a href="<?php echo basename(__FILE__); ?>?op=log&day=.">[&nbsp;Free Space Log&nbsp;(<?php echo $logsize_; ?>)&nbsp;]</a><br><br>
<table cols="9" rows="3" border="0">
<tr>
	<th>Worker</th>
<?php
for ($i = 0; $i < NUM_OF_COPIES; $i++)
	echo '	<th>' . ($i + 1) . "</th>\n";
?>	<th>0</th>
<?php
	if (NUM_OF_COPIES)
		echo <<<HTML
	<th>99</th>
HTML;
?></tr><tr>
	<td>Processing #</td>
<?php
foreach ($worker_status as $ws)
	echo "	<td align=\"center\">$ws</td>\n";
	if (NUM_OF_COPIES)
		echo <<<HTML
	<td align="center">--</td>
	<td align="center">--</td>
HTML;
?></tr><tr>
	<td>Today's Logs</td>
<?php
for ($i = 0; $i < NUM_OF_COPIES; $i++)
	echo '	<td align="center"><a href="' . basename(__FILE__) . '?op=log&wrk=' . ($i + 1) .
		'&day=0">&nbsp;' . $logsize0[$i] . "&nbsp;</a></td>\n";
?>	<td align="center">&nbsp;&nbsp;<a href="<?php echo basename(__FILE__); ?>?op=log&wrk=0&day=0">&nbsp;<?php echo $logsize0[NUM_OF_COPIES]; ?>&nbsp;</a></td>
<?php
	if (NUM_OF_COPIES)
	{
		echo <<<HTML
	<td align="center">&nbsp;&nbsp;<a href="
HTML;
		echo basename(__FILE__);
		echo <<<HTML
?op=log&wrk=99&day=0">&nbsp;
HTML;
		echo $logsize0[NUM_OF_COPIES + 1];
		echo <<<HTML
&nbsp;</a></td>
HTML;
	}
?></tr><tr>
	<td>Yesterday's Logs</td>
<?php
for ($i = 0; $i < NUM_OF_COPIES; $i++)
	echo '	<td align="center"><a href="' . basename(__FILE__) . '?op=log&wrk=' . ($i + 1) .
		'&day=1">&nbsp;' . $logsize1[$i] . "&nbsp;</a></td>\n";
?>	<td align="center">&nbsp;&nbsp;<a href="<?php echo basename(__FILE__); ?>?op=log&wrk=0&day=1">&nbsp;<?php echo $logsize1[NUM_OF_COPIES]; ?>&nbsp;</a></td>
<?php
	if (NUM_OF_COPIES)
	{
		echo <<<HTML
	<td align="center">&nbsp;&nbsp;<a href="
HTML;
		echo basename(__FILE__);
		echo <<<HTML
?op=log&wrk=99&day=1">&nbsp;
HTML;
		echo $logsize1[NUM_OF_COPIES + 1];
		echo <<<HTML
&nbsp;</a></td>
HTML;
	}
?></tr>
</table>
<br><?php
flush();		/* do this before the actual connection, so that the user at least sees the data above */
list($db, $err) = get_DB();
if (!$err)
{
	$num_of_entries = -1;

	$rs = $db->query('SELECT COUNT(*) FROM thn WHERE numoftries=0');
	if ($rs)
	{
		$row = $db->fetchNum($rs);
		echo 'In queue: ' . $row[0];
	}
	else
		echo 'query/1: ' . $db->getError();
	echo "<br>\n";

	$rs = $db->query("SELECT COUNT(*) FROM thn WHERE numoftries<>0");
	if ($rs)
	{
		$row = $db->fetchNum($rs);
		$num_of_entries = $row[0];
	}
	else
		echo 'query/2: ' . $db->getError();

	if (!$num_of_entries)
		echo "No deferred/stuck jobs\n";
	else
		if ($num_of_entries > 0)
		{
			echo "Deferred/stuck jobs: $num_of_entries<br>";

			$num_records_stmt = ' LIMIT 100';
			$query_prm_unlim = '';
			$unlimited = 0;					/* for groupAction() in JavaScript */
			if (isset($_REQUEST['all']))
				if (1 == $_REQUEST['all'])
				{
					$num_records_stmt = '';
					$query_prm_unlim = '&all=1';
					$unlimited = 1;
				}

			if (($num_of_entries > 100) && strlen($num_records_stmt))
				echo 'First 100 entries:<br>';

			$rs = $db->query('SELECT ' . F_IMAGE_UUID . ', path, xfersyntax, bitsstored, ' .
				'error, numoftries, nexttryat, worker' .
				' FROM thn WHERE numoftries<>0' .
				' ORDER BY numoftries DESC, nexttryat ASC, error DESC' . $num_records_stmt);
			if ($rs)
			{
echo <<<END
<table cols="9" border="1">
<colgroup>
	<col style="width: 1em" />
	<col />
	<col style="width: 11em" />
	<col style="width: 2em" />
	<col style="width: 2em" />
	<col style="width: 4em" />
	<col style="width: 9em" />
	<col style="width: 2em" />
	<col style="width: 5em" />
	<col style="width: 5em" />
	<col style="width: 5em" />
</colgroup>
<tr>
	<th>&times;</th>
	<th>uuid [path]</th>
	<th>xfersyntax</th>
	<th>bits</th>
	<th>error</th>
	<th>attempts</th>
	<th>next attempt</th>
	<th>wrk</th>
	<th>&nbsp;</th>
	<th>&nbsp;</th>
	<th>&nbsp;</th>
</tr>
END;
				while ($row = $db->fetchAssoc($rs))
				{
					echo "<tr bgColor=\"white\" onMouseOver=\"this.bgColor='gold';\" onMouseOut=\"this.bgColor='white';\">\n";
					echo '  <td><input name="uuid" type="checkbox" value="' . $row['uuid'] . "\"></td>\n";
					echo '	<td><a href="' . $row['path'] . '">' . $row['uuid'] . "</a></td>\n";
					echo '	<td>' . $row['xfersyntax'] . "</td>\n";
					$bs = $row['bitsstored'];
					if (empty($bs))
						$bs = '&nbsp;';
					echo "	<td align=\"right\">$bs</td>\n";
					echo '	<td align="left">' . $row['error'] . "</td>\n";
					echo '	<td align="right">' . $row['numoftries'] . "</td>\n";
					echo '	<td>' . $row['nexttryat'] . "</td>\n";
					echo '	<td align="center">' . $row['worker'] . "</td>\n";
					echo '	<td align="center"><a href="' . basename(__FILE__) . '?op=dump' . $query_prm_unlim .
						'&uid=' . $row['uuid'] . '">[&nbsp;Dump&nbsp;]</a>' . "</td>\n";
					echo '	<td align="center"><a href="' . basename(__FILE__) . '?op=retry' . $query_prm_unlim . '&uid=' .
						$row['uuid'] . '">[&nbsp;Retry&nbsp;]</a>' . "</td>\n";
					echo '	<td align="center"><a href="' . basename(__FILE__) . '?op=kill' . $query_prm_unlim . '&uid=' .
						$row['uuid'] . '">[&nbsp;Remove&nbsp;]</a>' . "</td>\n";
					echo "</tr>\n";
				}
			}
?>
</table><br>
<a href="javascript:groupSelection(1);">[&nbsp;Select visible&nbsp;]</a>&nbsp;&nbsp;
<a href="javascript:groupSelection(0);">[&nbsp;Unselect visible&nbsp;]</a>&nbsp;&nbsp;
<a href="javascript:groupAction('retry','<?php echo $unlimited; ?>');">[&nbsp;Retry selected&nbsp;]</a>&nbsp;&nbsp;
<a href="<?php echo basename(__FILE__) . '?op=retryall'; ?>">[&nbsp;Retry all failed&nbsp;]</a>&nbsp;&nbsp;
<a href="javascript:groupAction('kill','<?php echo $unlimited; ?>');">[&nbsp;Remove selected&nbsp;]</a>&nbsp;&nbsp;
<a href="javascript:groupAction('export','0');">[&nbsp;Export selected&nbsp;]</a><br>
<form method="post" name="groupActions" action="">
	<input id="groupInput" type="hidden" name="uid" value="">
	<input id="groupOp" type="hidden" name="op" value="">
	<input id="passUnlimited" type="hidden" name="all" value="">
</form>
<?php
			if (!$rs)
				echo 'query/3: ' . $db->getError();
		}

	$db->close();
}
else
	echo($err);
?>
</body></html>
